/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.util.*;
import socle.trame.*;
import socle.preferences.*;

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*-----------------------------------------------------------------------*/

/** IHM pour composer une liste ordonnee d'objets a partir de choix effectues dans une liste d'objets.
*/
public class ChooseFacetsPanel extends JPanel {

		private Facets                facettes;
		private ChooseObjectsPanel    choosePanel;
		private JComboBox             facetNames;
		private JTextField            facet;

	/** Creation de l'interface de selection des objets.
	*/
	public ChooseFacetsPanel(Object[] propertiesNames, Facets facets) {
		super(new BorderLayout());

		this.facettes = (Facets)facets.clone();
		final Object[] props = propertiesNames;

		Object[] dest = new Object[] {};
		choosePanel = new ChooseObjectsPanel(dest, true, dest);

		this.facetNames = new JComboBox(facettes.getNames());
		facetNames.setSelectedIndex(-1);
		facetNames.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					String facetName = (String)e.getItem();
					Object[] def = facettes.getFacetValue(facetName);
					choosePanel.setOrig(props);
					choosePanel.setDest(def);
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					String facetName = (String)e.getItem();
					facettes.setFacetValue(facetName, choosePanel.getSelected());
				}
			}
		});
		JPanel commands = new JPanel();
		this.facet = new JTextField(10);
		JButton addButton = new JButton("add");
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String facetName = facet.getText();
				if ((!facetName.equals("")) && (!contains(facetName))) {
					facetNames.addItem(facetName);
					facettes.addFacet(facetName, null);
					facetNames.setSelectedIndex(facetNames.getItemCount()-1);
					facet.setText("");
				}
			}
		});
		JButton delButton = new JButton("remove");
		delButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String facetName = facet.getText();
				if ((!facetName.equals("")) && (!facetName.equals(PreferencesModel.DEFAULT_FACET)) && (contains(facetName))) {
					facettes.removeFacet(facetName);
					facetNames.removeItem(facetName);
					facetNames.setSelectedIndex(0);
					facet.setText("");
				}
			}
		});
		commands.add(facet);
		commands.add(addButton);
		commands.add(delButton);
		commands.add(facetNames);

		add(commands, BorderLayout.NORTH);
		add(choosePanel, BorderLayout.CENTER);
	}
	private boolean contains(Object o) {
		for (int i=0; i<facetNames.getItemCount(); i++) {
			if (facetNames.getItemAt(i).equals(o)) {
				return true;
			}
		}
		return false;
	}
	/** Retourne la nouvelle valeur pour les facettes.
	*/
	public Facets getNewValue() {
		String facetName = (String)facetNames.getSelectedItem();
		if (facetName != null) {
			facettes.setFacetValue(facetName, choosePanel.getSelected());
		}
		return facettes;
	}
};
