/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*-----------------------------------------------------------------------*/

/** IHM pour composer une liste ordonnee d'objets a partir de choix effectues dans une liste d'objets.
*/
public class ChooseObjectsPanel extends JPanel {

	private boolean          with_litteral;
	//private DefaultListModel listModel_dest;
	private JList            list_orig;
	private JList            list_dest;

	/** Creation de l'interface de selection des objets.
	*/
	public ChooseObjectsPanel(Object[] orig, boolean with_litt, Object[] dest) {
		super(new BorderLayout());
		this.with_litteral = with_litt;

		DefaultListModel listModel_orig = new DefaultListModel();
		for (int i=0; i<orig.length; i++) {
			Object o = orig[i];
			if (!o.equals("TYPE"))
				listModel_orig.addElement(o);
		}
		this.list_orig = new JList(listModel_orig);
		list_orig.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list_orig.setLayoutOrientation(JList.VERTICAL);
		list_orig.setVisibleRowCount(-1);
		JScrollPane listScroller_orig = new JScrollPane(list_orig);
		listScroller_orig.setPreferredSize(new Dimension(250, 80));

		DefaultListModel listModel_dest = new DefaultListModel();
		for (int i=0; i<dest.length; i++) {
			Object o = dest[i];
			listModel_orig.removeElement(o);
			listModel_dest.addElement(o);
		}
		this.list_dest = new JList(listModel_dest);
		list_dest.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list_dest.setLayoutOrientation(JList.VERTICAL);
		list_dest.setVisibleRowCount(-1);
		JScrollPane listScroller_dest = new JScrollPane(list_dest);
		listScroller_dest.setPreferredSize(new Dimension(250, 80));

		final JTextField litteral = new JTextField(10);



		JPanel actions = new JPanel();
		actions.setLayout(new BoxLayout(actions, BoxLayout.Y_AXIS));
		JButton addButton = new JButton(">>");
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if ((with_litteral) && (!(litteral.getText().equals("")))) {
					Litteral o = new Litteral(litteral.getText());
					DefaultListModel listModel_dest = (DefaultListModel)list_dest.getModel();
					listModel_dest.addElement(o);
					litteral.setText("");
				} else {
					Object[] objects = list_orig.getSelectedValues();
					for (int i=0; i<objects.length; i++) {
						Object o = objects[i];
						DefaultListModel listModel_orig = (DefaultListModel)list_orig.getModel();
						DefaultListModel listModel_dest = (DefaultListModel)list_dest.getModel();
						listModel_dest.addElement(o);
						listModel_orig.removeElement(o);
					}
				}
			}
		});
		JButton delButton = new JButton("<<");
		delButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				Object[] objects = list_dest.getSelectedValues();
				for (int i=0; i<objects.length; i++) {
					Object o = objects[i];
					DefaultListModel listModel_orig = (DefaultListModel)list_orig.getModel();
					DefaultListModel listModel_dest = (DefaultListModel)list_dest.getModel();
					if (!(o instanceof Litteral)) listModel_orig.addElement(o);
					listModel_dest.removeElement(o);
				}
			}
		});

		if (with_litteral) {
			Dimension dim = litteral.getPreferredSize();
			dim.setSize(dim.getWidth(), delButton.getPreferredSize().getHeight());
			litteral.setPreferredSize(dim);

			Dimension max = litteral.getMaximumSize();
			max.setSize(max.getWidth(), delButton.getPreferredSize().getHeight());
			litteral.setMaximumSize(max);

			actions.add(litteral);
		}
		actions.add(addButton);
		actions.add(delButton);

		JPanel leftPanel = new JPanel(new BorderLayout());
		leftPanel.add(new JLabel(Textometre.LABELS.getLocaliseString("pref.free.property")), BorderLayout.NORTH);
		leftPanel.add(listScroller_orig, BorderLayout.CENTER);

		JPanel rightPanel = new JPanel(new BorderLayout());
		rightPanel.add(new JLabel(Textometre.LABELS.getLocaliseString("pref.choosen.property")), BorderLayout.NORTH);
		rightPanel.add(listScroller_dest, BorderLayout.CENTER);

		JPanel centerPanel = new JPanel(new BorderLayout());
		centerPanel.add(new JLabel(Textometre.LABELS.getLocaliseString("pref.action")), BorderLayout.NORTH);
		centerPanel.add(actions, BorderLayout.CENTER);

		add(leftPanel, BorderLayout.WEST);
		add(centerPanel, BorderLayout.CENTER);
		add(rightPanel, BorderLayout.EAST);
	}

	public void setOrig(Object[] orig) {
		DefaultListModel model = new DefaultListModel();
		for (int i=0; i<orig.length; i++) {
			Object o = orig[i];
			model.addElement(o);
		}
		list_orig.setModel(model);
	}
	public void setDest(Object[] dest) {
		DefaultListModel listModel_orig = (DefaultListModel)list_orig.getModel();
		DefaultListModel model = new DefaultListModel();
		for (int i=0; i<dest.length; i++) {
			Object o = dest[i];
			listModel_orig.removeElement(o);
			model.addElement(o);
		}
		list_dest.setModel(model);
	}

	/** Retourne les objets dans leur ordre de selection.
	*/
	public Object[] getSelected() {
		DefaultListModel listModel_dest = (DefaultListModel)list_dest.getModel();
		return listModel_dest.toArray();
	}
};
