/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.trame.*;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.event.EventListenerList;
/*-----------------------------------------------------------------------*/

/** Facettes des items.
* Il s'agit d'une table qui associe aux noms des facettes, leur definition sous forme
* d'un Vecteur d'objects (pour le moment String et Litteral)
*/
public class Facets implements Cloneable {

		private Hashtable<String, FacetDefinition> facets;
		private EventListenerList                  listeners;

	/** Cree un ensemble vide de facettes.
	*/
	public Facets() {
		this.facets = new Hashtable<String, FacetDefinition>();
		this.listeners = new EventListenerList();
	}
	/** Ajoute une facette avec sa definition.
	*/
	public void addFacet(String facetName, FacetDefinition facetValue) {
		if (facetValue == null) facetValue = new FacetDefinition();
		facets.put(facetName, facetValue);
	}
	/** Ajoute une facette dont la definition est egal au nom de la facette.
	*/
	public void addFacet(String propertyName) {
		FacetDefinition facetValue = new FacetDefinition();
		if (propertyName.equals(socle.preferences.PreferencesModel.DEFAULT_FACET)) {
			facetValue.add(Tnum.GRAPH);
		} else {
			facetValue.add(propertyName);
		}
		facets.put(propertyName, facetValue);
		fireFacetsChanged();
	}
	/** Supprime une facette.
	*/
	public void removeFacet(String facetName) {
		facets.remove(facetName);
		fireFacetsChanged();
	}
	/** Retourne la liste des noms de facettes.
	*/
	public String[] getNames() {
		String[] list = new String[facets.size()];
		int i = 0;
		for (Enumeration<String> e = facets.keys(); e.hasMoreElements(); ) {
			list[i++] = e.nextElement();
		}
		return list;
	}
	/** Retourne la definition d'une facette.
	*/
	public Object[] getFacetValue(String facetName) {
		FacetDefinition facetValue = facets.get(facetName);
		if (facetValue != null) {
			return facetValue.toArray();
		} else {
			return null;
		}
	}
	/** Retourne la definition d'une facette.
	*/
	public void setFacetValue(String facetName, Object[] facetValue) {
		if (facets.containsKey(facetName)) {
			FacetDefinition vector = new FacetDefinition();
			for (int i=0; i<facetValue.length; i++) {
				vector.add(facetValue[i]);
			}
			facets.put(facetName, vector);
			fireFacetsChanged();
		}
	}
	/** Efface toutes les facettes.
	*/
	public void clear() {
		facets.clear();
		fireFacetsChanged();
	}
	public void setModel(Facets f) {
		facets = f.facets;
		fireFacetsChanged();
	}
	public Object clone() {
	    Facets clone = new Facets();
		for (Enumeration<String> e = facets.keys(); e.hasMoreElements(); ) {
			String s = e.nextElement();
			FacetDefinition v = new FacetDefinition(facets.get(s));
			clone.addFacet(s,v);
		}
	    return clone;
	}
	public boolean equals(Facets o) {
		return facets.equals(o.facets);
	}

	/**
	 * Ajoute un FacetsListener
	 **/
	public void addFacetsListener(FacetsListener listener){
		listeners.add(FacetsListener.class, listener);
	}
	/**
	 * Supprime le FacetsListener
	 **/
	public void removeFacetsListener(FacetsListener l){
		 listeners.remove(FacetsListener.class, l);
	}
	/**
	 * Envoie un evenement de changement a tous les FacetsListeners
	 **/
	public void fireFacetsChanged() {
		FacetsListener[] listenerList = (FacetsListener[])listeners.getListeners(FacetsListener.class);
		for(int i=0; i<listenerList.length; i++) {
			FacetsListener listener = listenerList[i];
			listener.facetsChanged(new FacetsEvent(this, getNames()));
		}
	}

	/********************************************************************
	**   Definition d'une facette
	*********************************************************************/
	private class FacetDefinition extends Vector<Object> {
		public FacetDefinition() {
			super();
		}
		public FacetDefinition(FacetDefinition v) {
			super(v);
		}
	};
};
