/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.preferences.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.regex.*;
import javax.swing.text.*;

/*-----------------------------------------------------------------------*/

/**
 * Outil de recherche de chaine de caractere pour un editeur JTextPane.
 * Utilise les expression regulieres
 */
public class Find extends JDialog {

    private JTextPane        editor;
    private Document         doc;
    private JTextField       what;
    private JCheckBox        regex;
    private int              fromIndex;
    private Matcher          matcher;
    private JButton          findNext;

    public Find(JTextPane editor) {
		this.editor = editor;
		this.fromIndex = editor.getCaretPosition();
		this.what = new JTextField(20);
		what.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {setSearch();}
			public void insertUpdate(DocumentEvent e)  {setSearch();}
			public void removeUpdate(DocumentEvent e)  {setSearch();}
		});
        this.regex = new JCheckBox(Textometre.LABELS.getLocaliseString("winfind.useRegex"), false);
		regex.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				setSearch();
			}
		});
		JButton cancel = new JButton(Textometre.LABELS.getLocaliseString("cancel"));
        cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				dispose();
			}
		});
		findNext = new JButton(Textometre.LABELS.getLocaliseString("winfind.next"));
		findNext.setEnabled(false);
        findNext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				continueSearch();
			}
		});

        setModal(true);
        doc = editor.getDocument();;

        setResizable(false);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setTitle(Textometre.LABELS.getLocaliseString("winfind.title"));
        setSize(400, 100);
        getContentPane().setLayout(new BorderLayout());

        Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
        setLocation((int) ((screenSize.getWidth() - 400) / 2), (int) ((screenSize.getHeight() - 220) / 2));

        JPanel findOptions = new JPanel();
        findOptions.add(new JLabel(Textometre.LABELS.getLocaliseString("winfind.what")));
        findOptions.add(what);
        getContentPane().add(findOptions, BorderLayout.CENTER);

        JPanel findCommands = new JPanel(new FlowLayout(2, 3, 3));
        findCommands.add(regex);
        findCommands.add(findNext);
        findCommands.add(cancel);
        getContentPane().add(findCommands, BorderLayout.SOUTH);
    }

    private void setSearch() {
		String s = what.getText();
		boolean emptyString = s.equals("");
		findNext.setEnabled(!emptyString);
		if (!emptyString) {
			if (regex.isSelected()) {
				try {
					Pattern pattern = Pattern.compile(s);
					fromIndex = editor.getCaretPosition();
					String text = doc.getText(fromIndex, doc.getLength()-fromIndex);
					matcher = pattern.matcher(text);
				} catch(Exception e) {
					//System.err.println("Regex syntax error: "+e.getMessage());
				}
			} else {
				fromIndex = editor.getCaretPosition();
			}
		}
    }
    private void continueSearch() {
		if (regex.isSelected()) {
			if(matcher != null) {
				if(matcher.find()) {
					what.setBackground(Color.WHITE);
					try {
						editor.getCaret().setDot(matcher.start()+fromIndex);
						editor.getCaret().moveDot(matcher.end()+fromIndex);
						editor.getCaret().setSelectionVisible(true);
					} catch(Exception e) {
						System.err.println(e);
					}
					what.requestFocusInWindow();
				} else {
					what.setBackground(Color.RED);
					editor.setCaretPosition(0);
					setSearch();
				}
			}
		} else {
				try {
					String text = doc.getText(0, doc.getLength());
					String s = what.getText();
					int newIndex = text.indexOf(s, fromIndex);
					if(newIndex!=-1) {
						what.setBackground(Color.WHITE);
						try {
							editor.getCaret().setDot(newIndex);
							editor.getCaret().moveDot(newIndex+s.length());
							editor.getCaret().setSelectionVisible(true);
							fromIndex = newIndex+s.length();
						} catch(Exception e) {
							System.err.println(e.getMessage());
						}
						what.requestFocusInWindow();
					} else {
						what.setBackground(Color.RED);
						fromIndex = 0;
					}
				} catch (Exception e) {
					System.err.println(e.getMessage());
				}
		}
    }
}

