/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.trame.*;
import socle.tokenizer.*;
import socle.reader.*;

import java.nio.charset.Charset;
import java.io.File;
import java.io.FileOutputStream;
import java.io.StringReader;
import java.io.BufferedReader;

/*-----------------------------------------------------------------------*/

/**
 * Trameur.
 *
 * Programme de tramage d'une source textuelle.
 * La syntaxe d'appel au progamme est:
 * <code>socle.Trameur inputfile [-outputfile] [-charset] [-classtokenizer] [-separators]<code>
 **/
public class Trameur  {


	public static void main(String [] args) {
		if (args.length < 2) {
			System.out.println("Usage java socle.Trameur inputfile [-outputfile] [-charset] [-classtokenizer] [-separators]");
			System.exit(1);
		}
		Charset charset = Charset.defaultCharset();
		String classtokenizer = "socle.tokenizer.WordTokenizer";
		String separators = null;
		String outputfile = null;
        for (int i=1; i<args.length; i++) {
            String arg = args[i];
            if (arg.startsWith("-")) {
                String option = arg.substring(1);
                if (option.equals("charset")) {
					if (i+1 < args.length) {
						charset = Charset.forName(args[i+1]);
						i++;
					}
				} else if (option.equals("classtokenizer")) {
					if (i+1 < args.length) {
						classtokenizer = args[i+1];
						i++;
					}
				} else if (option.equals("separators")) {
					if (i+1 < args.length) {
						separators = args[i+1];
						i++;
					}
				} else if (option.equals("outputfile")) {
					if (i+1 < args.length) {
						outputfile = args[i+1];
						i++;
					}
				}
			}
		}
		try {
			TextDataReader reader = new TextDataReader(new File(args[0]), charset);
			BufferedReader dataReader = new BufferedReader(new StringReader(reader.getData()));

			DefaultTrameModel une_trame = new DefaultTrameModel();
			FileOutputStream stream = null;
			if (outputfile != null) {
				stream = new FileOutputStream(outputfile);
				byte[] content = new String("<?xml version='1.0' encoding='UTF-8'?><data xmlns='"+Textometre.TEXTOMETRE_NAMESPACE+"'>").getBytes("UTF-8");
				stream.write(content, 0, content.length);
				une_trame = new DefaultTrameModel(stream);
			}
			Tokenizer tokenizer = Tokenizer.newInstance(une_trame, classtokenizer, separators);
			if (tokenizer != null) {
					long debut = System.currentTimeMillis();
					tokenizer.tokenize(dataReader);
					dataReader.close();
					long tempsTotal = System.currentTimeMillis()-debut;
					System.out.println("tokens: " + (tokenizer.getItemPos()+1) +" Total time: "+tempsTotal+ " ms");
			} else {
				System.err.println("Erreur: le tokenizer n'a pas pu etre cree");
			}
			if (stream != null) {
				byte[] content = new String("</data>").getBytes("UTF-8");
				stream.write(content, 0, content.length);
			}
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
}
