/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.cadre;

import socle.*;
import socle.util.*;
import socle.trame.*;
import socle.selection.*;
import socle.partition.*;

import java.util.ArrayList;
import java.awt.Container;
/*-----------------------------------------------------------------------*/

/** Controlleur d'un cadre de navigation.
*/
public class CadreController {

	private CadreView             view = null;
	private CadreModel            model = null;

	/**
	 * Creation du controleur.
	 * @param model                Modele de donnees un cadre de navigation dans une trame textuelle.
	 **/
	public CadreController(CadreModel model) {
		this.model = model;
		view = new CadreView(this);
		model.addCadreListener(view);
	}

	/**
	 * Ajoute la vue dans le conteneur
	 * @param container               Le conteneur.
	 **/
	public void displayView(Container container) {
		container.add(view);
	}

	/**
	 * Affiche la zone dans la trame
	 **/
	public void showTrame(ZoneTrame sel) {
		model.fireCadreShowZone(sel);
	}

	/**
	 * Selectionne tous les elements dans la trame
	 **/
	public void selectTrame(ArrayList list, ZoneTrame zt) {
		model.fireCadreSelectSelection(list, zt);
	}

	/**
	 * Retourne vrai si il existe une selection de meme nom.
	 **/
	public boolean containsSelection(String title) {
		return model.selectionsContains(title);
	}

	/**
	 * Ajoute ou modifie une selection dans le cadre.
	 * @param selection     selection a ajouter ou a modifier
	 **/
	public void addSelection(AbstractSelection selection) {
		model.setSelection(selection);
	}

	/**
	 * Supprime une selection dans le cadre.
	 * @param selection     selection a supprimer
	 **/
	public void delSelection(AbstractSelection selection) {
		model.delSelection(selection);
	}

	public AbstractSelection getSelection(String title) {
		return model.getSelection(title);
	}

	/**
	 * Retourne vrai si il existe une partitions de meme nom.
	 **/
	public boolean containsPartition(String title) {
		return model.partitionsContains(title);
	}

	/**
	 * Ajoute ou modifie une partition dans le cadre.
	 * @param partition     partition a ajouter ou a modifier
	 **/
	public void addPartition(Partition partition) {
		model.setPartition(partition);
	}

	/**
	 * Suprime la partition du cadre.
	 **/
	public void delPartition(Partition partition) {
		model.delPartition(partition);
	}

	public Partition getPartition(String title) {
		return 	model.getPartition(title);
	}
}
