/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.conc;

import socle.*;
import socle.util.*;
import socle.trame.*;

/** Ligne d'une concordance.
*/
public class ConcLine extends Object {
       		private TrameModel   trame;
       		private Range        pivot;
       		private Range        leftContext;
       		private Range        rightContext;
       		private String       tnumName;

	/**
	 * Ligne de concordance.
	 * @param pivot       pivot
	 * @param trame       trame pour pouvoir convertir en chaines de caracteres
	 * @param contextSize nombre d'unites de contexte a retourner
	 **/
	public ConcLine(Range pivot, TrameModel trame, String tnumName, int contextSize) {
		this.trame = trame;
		this.pivot = pivot;
		this.tnumName = tnumName;
		this.leftContext = new Range(
			(pivot.getMinValue()-1-contextSize >= 0) ? pivot.getMinValue()-1-contextSize : 0,
			(pivot.getMinValue()-1 >= 0) ? pivot.getMinValue()-1: -1
		);
		this.rightContext = new Range(
			(pivot.getMaxValue()+1 < trame.size()) ? pivot.getMaxValue()+1: trame.size(),
			(pivot.getMaxValue()+contextSize+1 < trame.size()) ? pivot.getMaxValue()+contextSize+1 : trame.size() -1
		);
	}

	/**
	 * Retourne le contexte gauche.
	 **/
	public String getLeftContext() {
		return trame.getString(leftContext, tnumName);
	}

	/**
	 * Retourne le contexte droit.
	 **/
	public String getRightContext() {
		return trame.getString(rightContext, tnumName);
	}

	/**
	 * Retourne le pivot.
	 **/
	public String getPivot() {
		return trame.getString(pivot, tnumName);
	}

}