/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.conc;

import socle.*;
import socle.util.*;
import socle.selection.*;
import socle.preferences.*;
import socle.trame.*;
import socle.lexicon.article.*;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.EventListenerList;
/*-----------------------------------------------------------------------*/

/** Modele de donnees pour une concordance.
*/
public class ConcModel extends AbstractTableModel {

		private ArrayList             selection;
		private int                   contextSize;
		private String                fieldName;
		private Object[]              columnNames;
		private Object                data[][];
		private EventListenerList     listeners;

	/**
	 * Modele de donnees pour une concordance.
	 **/
	public ConcModel(int contextSize, String fieldName, ArrayList selection, TrameModel trame) {
		this.selection = selection;
		this.contextSize = contextSize;
		this.fieldName = fieldName;
		this.columnNames = new Object[] {"occ", "left context", "item", "right context"};
		this.data        = new Object[selection.size()][4];
		this.listeners = new EventListenerList();
		loadData(trame);
	}

	public int getColumnCount() {
		return data[0].length;
	}
	public int getRowCount() {
		return data.length;
	}
	public String getColumnName(int col) {
		return (String)columnNames[col];
	}
	public Object getValueAt(int row, int col) {
		return data[row][col];
	}
	public boolean isCellEditable(int row, int col) {
		if (col == 0) {
			return false;
		} else {
			return true;
		}
	}
	public Class getColumnClass(int col) {
		return getValueAt(0, col).getClass();
	}

	private void loadData(TrameModel trame) {
		for (int i=0; i<selection.size(); i++) {
			Range range = (Range)selection.get(i);

			ConcLine ic = new ConcLine(range, trame, fieldName, contextSize);
			data[i][0] = new GotoButton(range, i+1);
			data[i][1] = new ConcGraphPalindrome(ic.getLeftContext());
			data[i][2] = new ConcGraph(ic.getPivot());
			data[i][3] = new ConcGraph(ic.getRightContext());
		}
	}
	public void fireConcChanged(){
		ConcListener[] listenerList = (ConcListener[])listeners.getListeners(ConcListener.class);
		for(int i=0; i<listenerList.length; i++) {
			ConcListener listener = listenerList[i];
			listener.concChanged(new ConcChangedEvent(this));
		}
	}
	public void addConcListener(ConcListener listener){
		listeners.add(ConcListener.class, listener);
	}
	public void removeConcListener(ConcListener l){
		 listeners.remove(ConcListener.class, l);
	}
}