/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.conc;

import socle.*;
import socle.util.*;
import socle.cadre.*;
import socle.trame.*;
import socle.lexicon.article.*;
import socle.preferences.*;
import socle.selection.*;

import java.util.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import javax.swing.undo.*;

import javax.swing.text.JTextComponent;
import javax.swing.tree.TreePath;

/*-----------------------------------------------------------------------*/

/** Fenetre de concordancier.
*/
public class ConcView extends JInternalFrame implements ConcListener, PreferencesListener, TrameListener {

		private JTable                tableau;
		private JScrollPane           scroller;
		private String                fontName;
		private TrameCadreView        trameCadre;
		private TrameModel            trame;
		private ArrayList<Range>      selection;
		private int                   contextSize;
		private String                fieldName;
		private JLabel                findLabel, sizeLabel, fieldLabel;
		private JComboBox             fieldNameComboBox;
		private JMenuItem             addSelection;
		private String                messageNumberOfResponse, messageTooBig, messageAfficherLesPremiers, messageResponses;
		private CadreController       cadreController;

	/**
	 * Cree une fenetre dans laquelle pourra s'afficher les concordances.
	 **/
	public ConcView(TrameCadreView trameCadre) {
		super(Textometre.LABELS.getLocaliseString("windows.concordances"), true);
		Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
		setBounds((int)screenSize.getWidth() / 2,  (int)screenSize.getHeight() * 10 /100, ((int)screenSize.getWidth() / 2)- 10, (int)screenSize.getHeight() / 2);
		this.tableau = new JTable();
		this.trameCadre = trameCadre;
		this.trame = trameCadre.getTrame();
		this.selection = null;
		this.fontName = PreferencesModel.FONT_NAME;
		this.contextSize = PreferencesModel.CONC_CONTEXT_SIZE;
		this.fieldName = Tnum.GRAPH;
		this.messageNumberOfResponse = "message.number.responses.1";
		this.messageTooBig = "message.number.responses.2";
		this.messageAfficherLesPremiers = "message.number.responses.3";
		this.messageResponses = "message.number.responses.4";
		this.cadreController = trameCadre.getCadreController();
		DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
			public synchronized void drop (DropTargetDropEvent dropTargetDropEvent) {
				try {
					Transferable tr = dropTargetDropEvent.getTransferable();
					DataFlavor serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
					if (tr.isDataFlavorSupported (serialArrayListFlavor)) {
						dropTargetDropEvent.acceptDrop (DnDConstants.ACTION_COPY);
						ArrayList list = (ArrayList)tr.getTransferData(serialArrayListFlavor);
						AbstractSelection selAll = new BooleanSelection("title", trame.size());
						trame.setCurrentTnum(fieldName);
						for (int i=0; i<list.size(); i++) {
							Object o = list.get(i);
							if (o instanceof String) trame.setCurrentTnum(Tnum.GRAPH);
							AbstractSelection temp = trame.getOccs(o);
							selAll = selAll.union(temp);
						}
						showAsConc(selAll);
						dropTargetDropEvent.getDropTargetContext().dropComplete(true);
				  } else {
					System.err.println("Rejected");
					dropTargetDropEvent.rejectDrop();
				  }
				} catch (UnsupportedFlavorException e) {
					e.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				} catch (Exception err) {
					err.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				}
			}
		};
		DropTarget dropTarget = new DropTarget (this, dropTargetAdapter);

		//mise en place du menu
		Action actionSelection = new SetSelectionAction("actions.newSelection");
		this.addSelection = new JMenuItem(actionSelection);
		JMenu menu = new JMenu("Actions");
		menu.add(addSelection);
		JMenuBar mb = new JMenuBar();
		mb.add(menu);
		setJMenuBar(mb);

		java.text.NumberFormat format = java.text.NumberFormat.getInstance();
		format.setParseIntegerOnly(true);
		format.setMinimumIntegerDigits(0);
		format.setMaximumIntegerDigits(99);

		JFormattedTextField nb = new JFormattedTextField(format);
		nb.setColumns(2);
		nb.setValue(new Integer(contextSize));
		nb.addPropertyChangeListener("value", new java.beans.PropertyChangeListener() {
			public void propertyChange(java.beans.PropertyChangeEvent evt) {
				contextSize = ((Long)evt.getNewValue()).intValue();
			}
		});

		JTextField what = new JTextField("", 10);
		what.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//le tokenizer n'est pas le bon pour autre chose que le graph...
				trame.setCurrentTnum(Tnum.GRAPH);
				AbstractSelection liste = trame.getOccs(e.getActionCommand());
				showAsConc(liste);
			}
		});
		String[] fieldValues = {};
		fieldNameComboBox = new JComboBox(fieldValues);
		fieldNameComboBox.setSelectedItem(fieldName);
		fieldNameComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selectedIndex = fieldNameComboBox.getSelectedIndex();
				if (selectedIndex != -1) {
					fieldName = fieldNameComboBox.getItemAt(selectedIndex).toString();
				}
			}
		});

		scroller = new JScrollPane(tableau, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		JPanel cmds = new JPanel();
		this.findLabel = new JLabel("winconc.findLabel");
		this.sizeLabel = new JLabel("winconc.sizeLabel");
		this.fieldLabel = new JLabel("field.names");
		cmds.add(findLabel);
		cmds.add(what);
		cmds.add(sizeLabel);
		cmds.add(nb);
		cmds.add(fieldLabel);
		cmds.add(fieldNameComboBox);

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(cmds, BorderLayout.NORTH);
		panel.add(scroller, BorderLayout.CENTER);

		setContentPane(panel);
		//pack();
	}
	/**
	 * Construit une selection  partir des elements selectionnes dans le tableau de concordances ou bien.
	 * de tous les elements du tableau si aucune ligne n'est selectionnee
	 **/
	private AbstractSelection getSelectionSelected() {
		AbstractSelection sel = new IntervalSelection("title", trame.size());
		if (selection != null) {
			int[] selectedRows = tableau.getSelectedRows();
			int col = 0;
			for (int i=0; i<tableau.getColumnCount(); i++) {
				if (tableau.getColumnName(i).equals("occ")) {
					col = i;
				}
			}
			if (selectedRows.length > 0) {
				for (int i=0; i<selectedRows.length; i++) {
					GotoButton rowButton = (GotoButton)tableau.getValueAt(selectedRows[i], col);
					int row = rowButton.getNum() -1;
					sel.add(selection.get(row));
				}
			} else {
				for (int i=0; i<tableau.getRowCount(); i++) {
					GotoButton rowButton = (GotoButton)tableau.getValueAt(i, col);
					int row = rowButton.getNum() -1;
					sel.add(selection.get(row));
				}
			}
		}
		return sel;
	}

	/**
	 * Affiche une selection sous forme de concordances
	 **/
	private void showAsConc(AbstractSelection selAll) {
		ArrayList<Range> sel = selAll.toArray();

		//Limite l'affichage aux 5000 premieres reponses
		int answer = JOptionPane.OK_OPTION;
		if(sel.size() > 5000) {
			answer = JOptionPane.showConfirmDialog(null, messageNumberOfResponse+" ="+sel.size()+" "+messageTooBig+"\n"+messageAfficherLesPremiers+" 5000 "+messageResponses, JOptionPane.MESSAGE_PROPERTY, JOptionPane.YES_NO_OPTION);
			if (answer == JOptionPane.OK_OPTION) {
				sel = new ArrayList<Range>(sel.subList(0, 5000));
			} else {
				sel.clear();
			}
		}
		selection = sel;
		TableModel model = new ConcModel(contextSize, fieldName, sel, trame);

		if(model.getRowCount()!=0) {
			tableau = new JTable();
			tableau.addMouseListener(new GotoButtonListener(tableau));

			Font font = tableau.getFont();
			tableau.setFont(new Font(fontName, font.getStyle(), font.getSize()));

			myTableSorter sorter = new myTableSorter(model, tableau.getTableHeader());
			tableau.setModel(sorter);

			tableau.getTableHeader().setToolTipText("Click to specify sorting; Control-Click to specify secondary sorting");
			tableau.setDefaultRenderer(ConcGraphPalindrome.class, new PalindromeRenderer(fontName));
			tableau.setDefaultEditor(ConcGraphPalindrome.class, new ConcCellEditor(new JTextField(), JTextField.RIGHT));
			tableau.setDefaultEditor(ConcGraph.class,     new ConcCellEditor(new JTextField(), JTextField.LEFT));

			TableCellRenderer defaultRenderer = tableau.getDefaultRenderer(JButton.class);
			tableau.setDefaultRenderer(JButton.class, new GotoButtonRenderer(defaultRenderer));

            scroller.setViewportView(tableau);
			tableau.setShowGrid(false);

            TableColumnModel colModel = tableau.getColumnModel();
            int max = getMaxColumnWidth (2);
            int n = getMaxColumnWidth (0);
            int reste = ((getWidth() - max)  - n) / 2;
            colModel.getColumn(0).setPreferredWidth(n);
            colModel.getColumn(1).setPreferredWidth(reste);
            colModel.getColumn(2).setPreferredWidth(max);
            colModel.getColumn(3).setPreferredWidth(reste);
		} else {
			JOptionPane.showMessageDialog(null, messageNumberOfResponse+" = 0");
		}
	}

	/**
	 * Calule la taille maximale dont a besoin une colone.
	 * @param col     numero de la colone
	 **/
	private int getMaxColumnWidth(int col) {
			TableColumn column = tableau.getColumnModel().getColumn(col);
			Component comp = null;
			int maxWidth = 0;

        TableCellRenderer tableCellRenderer;
        int cellWidth   = 0;
        for (int i=0; i<tableau.getRowCount(); i++) {
            tableCellRenderer = tableau.getCellRenderer (i, col);
            comp = tableCellRenderer.getTableCellRendererComponent(tableau, tableau.getValueAt(i, col), false, false, i, col);
            if (comp instanceof JTextComponent) {
                JTextComponent jtextComp = (JTextComponent)comp;
                String text = jtextComp.getText();
                Font font = jtextComp.getFont();
                FontMetrics fontMetrics = jtextComp.getFontMetrics(font);
                int textWidth = SwingUtilities.computeStringWidth(fontMetrics, text);
                maxWidth = Math.max(maxWidth, textWidth);
            } else {
                cellWidth = comp.getPreferredSize().width;
                maxWidth = Math.max (maxWidth, cellWidth);
            }
        }
        return maxWidth + 4; //padding=4
    }

    public void concChanged(ConcChangedEvent event) {
		ConcModel model = (ConcModel)event.getSource();
	}

	/************************************************
	** Editeur pour une cellule de la table.
	** Ecriture impossible
	** PopUpMenu avec le bouton droit
	*************************************************/
	private class ConcCellEditor extends DefaultCellEditor implements TableCellEditor {
		public ConcCellEditor(JTextField textField, int direction) {
			super(textField);
			textField.setDragEnabled(true);

			setClickCountToStart(1);
			final Component comp = textField;
			Font font = textField.getFont();
			textField.setFont(new Font(fontName, Font.PLAIN, font.getSize()));
			textField.setHorizontalAlignment(direction);
			textField.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {e.consume();}
				public void keyPressed(KeyEvent e)  {e.consume();}
				public void keyTyped(KeyEvent e)    {e.consume();}
			});
			textField.addMouseListener(new MouseAdapter() {
				public void mouseReleased(MouseEvent e)  {
					if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
					}
				}
				public void mousePressed(MouseEvent e) {
					if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
					} else {
						try { //la chaine selectionnee est rendue transferable
							StringTransferHandler arrayListHandler = new StringTransferHandler();
							((JTextField)comp).setTransferHandler(arrayListHandler);
						} catch (Exception err) {
						}
					}
				}
			});
		}
	}
	/********************************************************************
	**   Afficheur pour le contexte gauche (aligne a droite)
	*********************************************************************/
	private class PalindromeRenderer extends JLabel implements TableCellRenderer {
		public PalindromeRenderer(String name) {
			this.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			setOpaque(true);
			Font font = getFont();
			setFont(new Font(name, Font.PLAIN, font.getSize()));
		}
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col){
			setText(value.toString());
			if (isSelected) {
				this.setBackground(table.getSelectionBackground());
			} else {
				this.setBackground(table.getBackground());
			}
			return this;
		}
	}
	private class GotoButtonRenderer implements TableCellRenderer {
			private TableCellRenderer afficheur;

		public GotoButtonRenderer(TableCellRenderer renderer) {
			afficheur = renderer;
		}

		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			if(value instanceof Component)
				return (Component)value;
			return afficheur.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		}
	}
	class GotoButtonListener implements MouseListener {
			private JTable theTable;

		public GotoButtonListener(JTable table) {
			theTable = table;
		}
		private void forwardEvent(MouseEvent e) {
			TableColumnModel columnModel = theTable.getColumnModel();
			int column = columnModel.getColumnIndexAtX(e.getX());
			int row    = e.getY() / theTable.getRowHeight();

			if(row >= theTable.getRowCount() || row < 0 || column >= theTable.getColumnCount() || column < 0)
				return;
			Object value = theTable.getValueAt(row, column);
			if(!(value instanceof GotoButton))
				return;
			GotoButton button = (GotoButton)value;
			MouseEvent buttonEvent = (MouseEvent)SwingUtilities.convertMouseEvent(theTable, e, button);
			button.dispatchEvent(buttonEvent);
			// This is necessary so that when a button is pressed and released
			// it gets rendered properly.  Otherwise, the button may still appear
			// pressed down when it has been released.
			if (e.getID() == MouseEvent.MOUSE_CLICKED) {
				Range range = button.getRange();
				cadreController.showTrame(new ZoneTrame(range.getMinValue(), range.getMaxValue()));
			}
			theTable.repaint();
		}
		public void mouseClicked(MouseEvent e) {forwardEvent(e);}
		public void mouseEntered(MouseEvent e) {forwardEvent(e);}
		public void mouseExited(MouseEvent e) {forwardEvent(e);}
		public void mousePressed(MouseEvent e) {forwardEvent(e);}
		public void mouseReleased(MouseEvent e) {forwardEvent(e);}
	}
	/********************************************************************
	**   Action d'ajout ou de modification de selection dans le cadre
	*********************************************************************/
	public class SetSelectionAction extends AbstractAction {
		public SetSelectionAction(String message) {
			super(message);
		}
		public void addContent(AbstractSelection sel) {
			cadreController.addSelection(sel);
		}
		public void removeContent(AbstractSelection sel) {
			cadreController.delSelection(sel);
		}
		public void actionPerformed(ActionEvent e) {
			AbstractSelection sel = trameCadre.getNewSelection(getSelectionSelected());
			if (sel != null) {
				if (cadreController.containsSelection(sel.getTitle())) {
					AbstractSelection sel_old = cadreController.getSelection(sel.getTitle());
					UndoableEdit edit = new SelectionEdit(this, sel_old, sel);
					Textometre.UNDO_SUPPORT.postEdit(edit);
				} else {
					UndoableEdit edit = new SelectionEdit(this, null, sel);
					Textometre.UNDO_SUPPORT.postEdit(edit);
				}
			}
		}
	}
	/********************************************************************
	**   Action reversible d'ajout / remplacement de selection
	*********************************************************************/
	public class SelectionEdit extends AbstractUndoableEdit {

			private SetSelectionAction action;
			private AbstractSelection  sel_old;
			private AbstractSelection  sel_new;

		public SelectionEdit(Object obj, AbstractSelection sel_old, AbstractSelection sel_new) {
			this.action = (SetSelectionAction)obj;
			this.sel_new = sel_new;
			this.sel_old = sel_old;
			if (sel_old!=null) action.removeContent(sel_old);
			action.addContent(sel_new);
		}
		public void undo() throws CannotUndoException {
			super.undo();
			action.removeContent(sel_new);
			if (sel_old!=null) action.addContent(sel_old);
		}
		public void redo() throws CannotRedoException {
			super.redo();
			if (sel_old!=null) action.removeContent(sel_old);
			action.addContent(sel_new);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			if (sel_old!=null) return "remplace selection";
			return "append selection";
		}
	}
	/********************************************************************
	**   Ecoute les evenements de modifications en provenance de la trame
	*********************************************************************/
	public void trameChanged(TrameEvent event) {
		if (event instanceof TrameFieldsChangedEvent) {
			TrameFieldsChangedEvent e = (TrameFieldsChangedEvent)event;
			fieldName = (String)fieldNameComboBox.getSelectedItem();
			fieldNameComboBox.removeAllItems();

			ArrayList<String> list = e.getNames();
			for (int i=0; i<list.size(); i++) {
				fieldNameComboBox.addItem(list.get(i));
			}
			if (fieldName == null) fieldName = Tnum.GRAPH;
			fieldNameComboBox.setSelectedItem(fieldName);
		}
	}
	/********************************************************************
	**   Ecoute les evenements de modifications des preferences
	*********************************************************************/
	public void preferencesChanged(PreferencesChangedEvent event) {
		if (event instanceof PreferencesChangedFontEvent) {
			PreferencesChangedFontEvent e = (PreferencesChangedFontEvent)event;
			fontName = e.getNewFontName();
			Font font = tableau.getFont();
			tableau.setFont(new Font(fontName, font.getStyle(), font.getSize()));
			tableau.setDefaultRenderer(ConcGraphPalindrome.class, new PalindromeRenderer(fontName));
			tableau.updateUI();
		} else if (event instanceof PreferencesChangedLocaleEvent) {
			this.findLabel.setText(Textometre.LABELS.getLocaliseString("winconc.findLabel"));
			this.sizeLabel.setText(Textometre.LABELS.getLocaliseString("winconc.sizeLabel"));
			this.fieldLabel.setText(Textometre.LABELS.getLocaliseString("field.names"));
			this.messageNumberOfResponse = Textometre.LABELS.getLocaliseString("message.number.responses.1");
			this.messageTooBig = Textometre.LABELS.getLocaliseString("message.number.responses.2");
			this.messageAfficherLesPremiers = Textometre.LABELS.getLocaliseString("message.number.responses.3");
			this.messageResponses = Textometre.LABELS.getLocaliseString("message.number.responses.4");
			addSelection.setText(Textometre.LABELS.getLocaliseString("actions.newSelection"));
		}
	}
}