/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon.article;

import java.io.*;
/*-----------------------------------------------------------------------*/

/** La classe <code>Article</code> definit un item lexical.
*
* <p>Cette classe sert a representer des items lexicaux. Un item lexical se caracterise par:
* sa forme graphique et son statut (lexical ou separateur)
* Il peut se voir affecter un numero d'ordre pouvant servir d'identifiant, une frequence
* et des rangs lexicometriques et lexicographiques.</p>
*/
public class Article implements Serializable {
		private String  graph;
		private int     lexicom;
		private int     lexicog;
		private long    freq;
		private int     numap;

		public static char       LEXICAL   =  '0';
		public static char       SEPARATOR =  '1';
		public static char       NEWLINE   = '\n';

	/** Initialise un nouvel article.
	*
	* @param graph     Representation graphique de l'article.
	* @param numap     Numero d'ordre de l'article.
	*/
	public Article(String graph, int numap) {
		this.graph   = graph;
		this.lexicom = 0;
		this.lexicog = 0;
		this.freq    = 1;
		this.numap   = numap;
	}
	/** Retourne le numero d'ordre de l'article.
    *
    * @return numero d'ordre
	*/
	public int getNumap() {
		return numap;
	}
	/** Retourne la frequence de l'article.
    *
    * @return frequence
	*/
	public long getFreq() {
		return freq;
	}
	/** Fixe la frequence de l'article.
	*/
	public void setFreq(long freq) {
		this.freq = freq;
	}
	/** Retourne la forme graphique de l'article.
    *
    * @return graphie
	*/
	public String getGraph() {
		return graph;
	}
	/** Fixe la forme graphique de l'article.
    *
    * @param graph graphie
	*/
	public void setGraph(String graph) {
		this.graph   = graph;
	}
	/** Fixe le rang lexicometrique de l'article.
	* @param rang     rang lexicometrique.
	*/
	public void setLexicom(int rang) {
		lexicom = rang;
	}
	/** Fixe le rang lexicographique de l'article.
	* @param rang     rang lexicographique.
	*/
	public void setLexicog(int rang) {
		lexicog = rang;
	}
	/** Incremente la frequence de l'article.
	*/
	public void incrFreq() {
		freq++;
	}
	/**
	 * Retourne une reprsentation sous forme de chaine de caractre.utile pour debuger.
	 **/
	public String toString() {
		return "[article "
		+" freq:"+String.valueOf(freq)
		+" lexicom:"+String.valueOf(lexicom)
		+" lexicog:"+String.valueOf(lexicog)
		+" freq:"+String.valueOf(freq)
		+" numap:"+String.valueOf(numap)
		+" graph:"+graph
		+ "]";
	}
}
