/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon.article;

import java.util.*;
import java.awt.*;
import java.awt.datatransfer.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
/*-----------------------------------------------------------------------*/

/** Classe utilisee pour le transfert de donnees de type Article vers un composant Swing.
*/
public class ArticleTransferHandler extends TransferHandler {
	DataFlavor localArrayListFlavor, serialArrayListFlavor;
	String localArrayListType = DataFlavor.javaJVMLocalObjectMimeType + ";class=java.util.ArrayList";
	JTable source = null;

	public ArticleTransferHandler() {
		try {
			localArrayListFlavor = new DataFlavor(localArrayListType);
		} catch (ClassNotFoundException e) {
			System.err.println("ArticleTransferHandler: unable to create data flavor");
		}
		serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
	}
	private boolean hasLocalArrayListFlavor(DataFlavor[] flavors) {
		if (localArrayListFlavor == null) {
			return false;
		}
		for (int i = 0; i < flavors.length; i++) {
			if (flavors[i].equals(localArrayListFlavor)) {
				return true;
			}
		}
		return false;
	}
	private boolean hasSerialArrayListFlavor(DataFlavor[] flavors) {
		if (serialArrayListFlavor == null) {
			return false;
		}
		for (int i = 0; i < flavors.length; i++) {
			if (flavors[i].equals(serialArrayListFlavor)) {
				return true;
			}
		}
		return false;
	}
	public boolean canImport(JComponent c, DataFlavor[] flavors) {
		if (hasLocalArrayListFlavor(flavors))  { return true; }
		if (hasSerialArrayListFlavor(flavors)) { return true; }
		return false;
	}
	protected Transferable createTransferable(JComponent c) {
		if (c instanceof JTable) {
			source = (JTable)c;
			int[] values = source.getSelectedRows();
			if (values == null || values.length == 0) {
				return null;
			}
			ArrayList<Article> alist = new ArrayList<Article>(values.length);
			for (int i = 0; i < values.length; i++) {
				int row = (int)values[i];
				Graph graph = (Graph)source.getValueAt(row, 0);
				alist.add(graph.getArticle());
			}
			return new ArrayListTransferable(alist);
		}
		return null;
	}
	public int getSourceActions(JComponent c) {
		return COPY;
	}

	/*********************************
	**
	**********************************/
	public class ArrayListTransferable implements Transferable {
		ArrayList data;

		public ArrayListTransferable(ArrayList alist) {
			data = alist;
		}
		public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
			if (!isDataFlavorSupported(flavor)) {
				throw new UnsupportedFlavorException(flavor);
			}
			return data;
		}
		public DataFlavor[] getTransferDataFlavors() {
			return new DataFlavor[] { localArrayListFlavor, serialArrayListFlavor };
		}
		public boolean isDataFlavorSupported(DataFlavor flavor) {
			if (localArrayListFlavor.equals(flavor)) {
				return true;
			}
			if (serialArrayListFlavor.equals(flavor)) {
				return true;
			}
			return false;
		}
	}
}
