/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon.article;

import socle.*;

import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
/*-----------------------------------------------------------------------*/

/** Fenetre de dialogue pour consulter ou modifier les proprietes d'une occurrence.
*/
public class OccurrenceView extends JDialog {

	/**
	 * Fenetre de dialogue pour consulter ou modifier les proprietes d'une occurrence.
	 **/
	public OccurrenceView(OccurrenceController c) {
 		super();
 		setTitle(Textometre.LABELS.getLocaliseString("action.edit.occurrence"));
 		final OccurrenceController controller = c;
		final HashMap<String,String> properties = new HashMap<String,String>(controller.getProperties());

        Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
        setLocation((int) ((screenSize.getWidth() - 400) / 2), (int) ((screenSize.getHeight() - 220) / 2));
 		this.setModal(true);
 		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

		JButton okButton           = new JButton("ok");
		JButton cancelButton       = new JButton("cancel");
		JPanel  cmds               = new JPanel();
		cmds.add(okButton);
		cmds.add(cancelButton);
		okButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				controller.setProperties(properties);
				dispose();
			}
		});
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});

		getContentPane().setLayout(new GridLayout(0,1));
		Set propertieNames = properties.keySet();
		Iterator iterator = propertieNames.iterator();
		while (iterator.hasNext()) {
			final String name = (String)iterator.next();
			String value = controller.getValue(name);
			JPanel prop = new JPanel(new GridLayout(1,2, 4,1));
			JTextField edit = new JTextField(value);
			edit.getDocument().addDocumentListener(new DocumentListener() {
				public void changedUpdate(DocumentEvent e) {
					try {
						properties.put(name, e.getDocument().getText(0, e.getDocument().getLength()));
					} catch (Exception err) {
						err.printStackTrace();
					}
				}
				public void insertUpdate(DocumentEvent e)  {
					try {
						properties.put(name, e.getDocument().getText(0, e.getDocument().getLength()));
					} catch (Exception err) {
						err.printStackTrace();
					}
				}
				public void removeUpdate(DocumentEvent e)  {
					try {
						properties.put(name, e.getDocument().getText(0, e.getDocument().getLength()));
					} catch (Exception err) {
						err.printStackTrace();
					}
				}
			});
			prop.add(new JLabel(name, JLabel.RIGHT));
			prop.add(edit);
			getContentPane().add(prop);
		}

 		getContentPane().add(cmds);
		pack();
	}
}
