/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.map;

import socle.*;
import socle.util.*;
import socle.selection.*;
import socle.partition.*;
import socle.trame.*;
import socle.cadre.*;
import socle.preferences.*;
import socle.lexicon.article.*;

import java.util.ArrayList;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.event.*;
/*-----------------------------------------------------------------------*/

/** Carte des sections.
*/
public class MapView extends JInternalFrame implements CadreListener, PreferencesListener {

		//visualisateur de sections
		private JPanel                browser;

		//la carte des sections
		private JPanel                map;

		//menu des partitions disponibles
		private JComboBox             menu;

		private TrameModel             trame;
		private CadreModel             cadre;
		private Partition              current_partition;
		private ArrayList<MyCheckBox>  current_elements;
		private Color                  bgcolor;

	/**
	 * Editeur qui affiche la carte des sections.
	 **/
	public MapView(TrameModel the_trame, CadreModel the_cadre) {
		super("carte des sections", true);
		this.trame = the_trame;
		this.cadre = the_cadre;
		this.current_partition = new Partition("empty");
		this.current_elements = new ArrayList<MyCheckBox>();
		this.bgcolor = PreferencesModel.BG_COLOR;

		this.menu = new JComboBox(new Object[] {"structure"});
		JButton reload =  new JButton(new ImageIcon(getClass().getResource("/icons/reload.png")));
		reload.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				reloadModel();
			}
		});

		this.map = new JPanel();
		DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
			public synchronized void drop (DropTargetDropEvent dropTargetDropEvent) {
				try {
					Transferable tr = dropTargetDropEvent.getTransferable();
					DataFlavor serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
					if (tr.isDataFlavorSupported (serialArrayListFlavor)) {
						dropTargetDropEvent.acceptDrop (DnDConstants.ACTION_COPY);
						ArrayList list = (ArrayList)tr.getTransferData(serialArrayListFlavor);
						AbstractSelection selectionAll = new BooleanSelection(trame.size());
						for (int i=0; i<list.size(); i++) {
							Object o = list.get(i);
							if (o instanceof Article) {
								AbstractSelection tmp = trame.getOccs((Article)o);
								selectionAll = selectionAll.union(tmp);
							}
						}
						hilite(selectionAll);
						dropTargetDropEvent.getDropTargetContext().dropComplete(true);
				  } else {
					System.err.println("Rejected");
					dropTargetDropEvent.rejectDrop();
				  }
				} catch (UnsupportedFlavorException e) {
					e.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				} catch (Exception err) {
					err.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				}
			}
		};
		DropTarget dropTarget = new DropTarget(map, dropTargetAdapter);

		this.browser = new JPanel(new FlowLayout(FlowLayout.LEFT));

		JScrollPane map_scroller = new JScrollPane(map, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollPane browser_scroller = new JScrollPane(browser, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JSplitPane splitPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, map_scroller, browser_scroller);
		JPanel content = new JPanel(new BorderLayout());
		JPanel commands = new JPanel();
		commands.add(menu);
		commands.add(reload);
		content.add(commands, BorderLayout.NORTH);
		content.add(splitPanel, BorderLayout.CENTER);
		setContentPane(content);
	}
	/**
	 * Colorie les partie contenant dles zones de la carte textuelle.
	 * @param selection     selection d'unites dans la trame
	 **/
	private void hilite(AbstractSelection selection) {
		ArrayList tableau = selection.toArray();
		for (int i=0; i<tableau.size(); i++) {
			Range range = (Range)tableau.get(i);
			ZoneTrame zone = new ZoneTrame(range.getMinValue(), range.getMaxValue());
			ArrayList<ZoneTrame> liste = current_partition.getPartition();//c'est assez mal nomme  getPartition
			for (int j=0; j<liste.size(); j++) {
				ZoneTrame zt = liste.get(j);
				if (zt.contains(zone)) {
					try {
						MyCheckBox comp = current_elements.get(j);
						comp.setBackground(bgcolor);
					} catch (Exception err) {
						err.printStackTrace();
					}
				}
			}
		}
	}

	private void reloadModel () {
		String selected = (String)menu.getSelectedItem();
		if (selected.equals("structure")) {
			loadModel(cadre.getStructure());
		} else {
			loadModel(cadre.getPartition(selected));
		}
	}
	private void loadModel(org.w3c.dom.Node node) {
		browser.removeAll();
		browser.updateUI();

		map.removeAll();
		Box box = new Box(BoxLayout.Y_AXIS);
		map.add(box);
		recurs(node, box);
		map.updateUI();
	}
	private void loadModel(Partition part) {
		current_partition = part;
		current_elements.clear();
		browser.removeAll();
		browser.updateUI();
		map.removeAll();
		JPanel panel_node = new JPanel(new GridLayout(0, 50, 2, 2));
		for (int i=0; i<part.size(); i++) {
			final ZoneTrame zt = part.getZone(i);
			final MyCheckBox checkbox = new MyCheckBox();
			checkbox.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					checkbox.changeSelected();
					if (checkbox.isSelected()) {
						checkbox.setText(zt);
						browser.add(checkbox.getEditor());
						browser.setPreferredSize(maxDim());
						browser.updateUI();
					} else {
						browser.remove(checkbox.getEditor());
						browser.setPreferredSize(maxDim());
						browser.updateUI();
					}
				}
			});
			panel_node.add(checkbox);
			current_elements.add(checkbox);
		}

		JPanel panel_num = new JPanel(new GridLayout(0, 1, 2, 2));
		for (int i=0; i<=(part.size() / 50); i++) {
			JLabel label = new JLabel(String.valueOf(50*(i+1)));
			label.setPreferredSize(new Dimension((int)label.getPreferredSize().getWidth(), 12));
			panel_num.add(label);
		}

		panel_node.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.LineBorder(Color.red, 1, true), part.toString()));
		map.add(panel_node);
		panel_num.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.LineBorder(Color.red, 1, true), part.toString()));
		map.add(panel_num);
		map.updateUI();
	}
	private Dimension maxDim() {
		double maxWidth = 0;
		double maxHeight = 0;
		Component[] components = browser.getComponents();
		for (int i=0; i<components.length ; i++) {
			double width = components[i].getPreferredSize().getWidth();
			double height = components[i].getY();
			if (width > maxWidth) {
				maxWidth = width;
			}
			if (height > maxHeight) {
				maxHeight = height;
			}
		}
		return new Dimension((int)(maxWidth+10), (int)(maxHeight+20));
	}

	private void recurs(org.w3c.dom.Node node, Box panel_parent) {
		Box panel_node = new Box(BoxLayout.PAGE_AXIS);
		panel_node.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.LineBorder(Color.red, 1, true), node.getNodeName()));
		if (node.getNodeName().equals("text")) {
			panel_node = panel_parent;
		}
		org.w3c.dom.NamedNodeMap attributes = node.getAttributes();
		if (attributes != null ) {
			String start = null;
			String end = null;
			if ((attributes.getNamedItem("start")!=null) && (attributes.getNamedItem("end")!=null)) {
				start = attributes.getNamedItem("start").getNodeValue().trim();
				end = attributes.getNamedItem("end").getNodeValue().trim();
			} else if ((attributes.getNamedItem("from")!=null) && (attributes.getNamedItem("until")!=null)) {
				start = attributes.getNamedItem("from").getNodeValue().trim();
				end = attributes.getNamedItem("until").getNodeValue().trim();
			}
			if ((start != null) && (end != null)) {
				final ZoneTrame zone = new ZoneTrame(start, end);
				final JPanel p = new JPanel(new FlowLayout(FlowLayout.LEFT));
				if (!node.hasChildNodes()) {
					for (int i=zone.getMinValue(); i<=zone.getMaxValue(); i++) {
						final ZoneTrame zt = new ZoneTrame(i, i);
						final MyCheckBox checkbox = new MyCheckBox();
						checkbox.addMouseListener(new MouseAdapter() {
							public void mouseClicked(MouseEvent e) {
								checkbox.changeSelected();
								if (checkbox.isSelected()) {
									checkbox.setText(zt);
									browser.add(checkbox.getEditor());
									browser.setPreferredSize(maxDim());
									browser.updateUI();
								} else {
									browser.remove(checkbox.getEditor());
									browser.setPreferredSize(maxDim());
									browser.updateUI();
								}
							}
						});
						p.add(checkbox);
					}
				}
				panel_node.add(p);
			}
		}
		if (node.hasChildNodes()) {
			org.w3c.dom.NodeList nodelist = node.getChildNodes();
			for (int i=0; i<nodelist.getLength(); i++) {
				org.w3c.dom.Node child = nodelist.item(i);
				recurs(child, panel_node);
			}
		} else {
		}
		panel_node.add(Box.createHorizontalStrut(650));
		if (!node.getNodeName().equals("text")) panel_parent.add(panel_node);
	}

	/********************************************************************
	**   Ecoute les evenements de modifications en provenance du cadre
	*********************************************************************/
	public void cadreChanged(CadreEvent event) {
		if (event instanceof PartitionsChangedEvent) {
			PartitionsChangedEvent e = (PartitionsChangedEvent)event;
			Object selected = menu.getSelectedItem();
			menu.removeAllItems();
			ArrayList list = e.getPartitionNames();
			menu.addItem("structure");
			for (int i=0; i<list.size(); i++) {
				menu.addItem((String)list.get(i));
			}
			if (list.contains(selected)) menu.setSelectedItem(selected);
		}
	}
	/********************************************************************
	**   Ecoute les evenements de modifications en provenance des preferences
	*********************************************************************/
	public void preferencesChanged(PreferencesChangedEvent event) {
		if(event instanceof PreferencesChangedBgColorEvent) {
			PreferencesChangedBgColorEvent e = (PreferencesChangedBgColorEvent)event;
			bgcolor = e.getNewBgColor();
		}
	}
    /**********************************************************
    ****                    Helper classes
    ***********************************************************/
    private class MyCheckBox extends JPanel {
			private JTextField editor;
			private boolean selected;

        public MyCheckBox() {
			super();
			setBorder(new javax.swing.border.LineBorder(Color.black, 2, true));
            this.editor = new JTextField();
            this.selected = false;
			setPreferredSize(new Dimension(12, 12));
        }
        public JTextField getEditor() {
			return editor;
		}
		public void setText(ZoneTrame zone) {
			editor.setText(trame.getString(new Range(zone.getMinValue(), zone.getMaxValue()), Tnum.GRAPH));
		}
		public void setBackground(Color c) {
			super.setBackground(c);
		}
		public boolean isSelected() {
			return selected;
		}
		public void setSelected(boolean b) {
			selected = b;
		}
		public void changeSelected() {
			setSelected(!isSelected());
			updateUI();
		}
		public void paint (Graphics g) {
			super.paint(g);
			if (isSelected()) {
				g.drawLine(0, 0, getWidth()-1, getHeight()-1);
				g.drawLine(0, getHeight()-1, getWidth()-1, 0);
			} else {
			}
		}
    }
}