/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.partition;

import socle.*;
import socle.util.*;

import java.util.*;
import org.w3c.dom.*;

/*-----------------------------------------------------------------------*/

/** Decoupage d'une trame textuelle en parties. La somme des parties est egale a la trame dans sa totalite.
*/
public class Partition {

		ArrayList<ZoneTrame>   partition;
		String                 title;

	/**
	 * Partition .
	 * @param title      Nom de la partition.
	 **/
	public Partition(String title) {
		this.title = title;
		this.partition = new ArrayList<ZoneTrame>();
	}
	/**
	 * Construit une nouvelle partition vide sans titre.
	 **/
	public Partition() {
		this("");
	}
	public Partition(Element elt) {
		this.partition = new ArrayList<ZoneTrame>();
		NodeList titles = elt.getElementsByTagName("title");
		Element titleElt = (Element)titles.item(0);
		this.title = titleElt.getTextContent();

		NodeList zones = elt.getElementsByTagName("zone");
		for (int i=0; i<zones.getLength(); i++) {
			Element zone = (Element)zones.item(i);
			String from = zone.getAttribute("from");
			String until = zone.getAttribute("until");
			ZoneTrame zoneTrame = new ZoneTrame(from, until);
			add(zoneTrame);
		}
	}
	/**
	 * Ajoute une nouvelle zone a la partition.
	 **/
	public void add(ZoneTrame zone) {
		partition.add(zone);
	}
	/**
	 * Fixe la liste des parties.
	 **/
	public void setPartition(ArrayList<ZoneTrame> partition) {
		this.partition = partition;
	}
	/**
	 * Fixe le nom de la partition.
	 **/
	public void setTitle(String title) {
		this.title = title;
	}
	/**
	 * Recupere la liste des parties.
	 **/
	public ArrayList<ZoneTrame> getPartition() {
		return partition;
	}
	/**
	 * Recupere le nom de la partition.
	 **/
	public String getTitle() {
		return title;
	}
	/**
	 * Retourne la nieme zone de la selection.
	 **/
	public ZoneTrame getZone(int position) {
		return (ZoneTrame)partition.get(position);
	}
	/**
	 * Retourne le nombre de zones de la selection.
	 **/
	public int size() {
		return partition.size();
	}
	public String toXML() {
		String titleXML = XMLString.toString(getTitle());
		String resultat = "<partition>";
		resultat += "<title>"+titleXML+"</title>";
		for (int i=0; i<size(); i++) {
			ZoneTrame zone = getZone(i);
			resultat += "<zone from='"+zone.getMinValue()+"' until='"+zone.getMaxValue()+"'/>";
		}
		return resultat+"</partition>";
	}
	public String toString() {
		return title;
	}
}