/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.preferences;

import socle.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import java.util.jar.*;
/*-----------------------------------------------------------------------*/

/** Fenetre de dialogue pour changer / consulter les preferences de decoupage en unites.
*/
public class DialogTokenizer extends JDialog {

	private PreferencesController preferencesController;
	private JComboBox             listeTokenizer;
	private JTextField            strTokenizer;

	public DialogTokenizer(PreferencesController controller) {
 		super();
        Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
        setLocation((int) ((screenSize.getWidth() - 400) / 2), (int) ((screenSize.getHeight() - 220) / 2));
 		this.setModal(true);
 		this.preferencesController = controller;

		JButton okButton           = new JButton(Textometre.LABELS.getLocaliseString("ok"));
		JButton cancelButton       = new JButton(Textometre.LABELS.getLocaliseString("cancel"));
		JPanel  cmds               = new JPanel();
		cmds.add(okButton);
		cmds.add(cancelButton);
		okButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				valid();
			}
		});
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		String str = preferencesController.getStrTokenizer();
		str = str.replaceAll("\t", "\\\\t");
		str = str.replaceAll("\n", "\\\\n");
		str = str.replaceAll("\r", "\\\\r");
		strTokenizer = new JTextField(str);
		listeTokenizer = new JComboBox();
		try{
			Class parent = Class.forName("socle.tokenizer.Tokenizer");
			java.util.List<Class> classes = getClasses("socle.tokenizer");

			// normalement les classes sont trouvees automatiquement par getClasses mais avec java Webstart
			// la propriete java.class.path ne retourne que jre.../lib/deploy.jar
			Class cl;
			cl = Class.forName("socle.tokenizer.CharTokenizer");
			if (!classes.contains(cl)) {
				classes.add(cl);
			}
			cl = Class.forName("socle.tokenizer.WordTokenizer");
			if (!classes.contains(cl)) {
				classes.add(cl);
			}
			cl = Class.forName("socle.tokenizer.SeparatorTokenizer");
			if (!classes.contains(cl)) {
				classes.add(cl);
			}
			cl = Class.forName("socle.tokenizer.RegexTokenizer");
			if (!classes.contains(cl)) {
				classes.add(cl);
			}

			for (int i=0; i<classes.size(); i++) {
				Class sup = ((Class)classes.get(i)).getSuperclass();
				if (sup.equals(parent)) {
					listeTokenizer.addItem(((Class)classes.get(i)).getName());
				}
			}
		} catch (Exception err) {
			System.err.println(err.getMessage());
		}
		listeTokenizer.setSelectedItem(preferencesController.getTokenizer().getName());

 		getContentPane().setLayout(new GridLayout(3,2, 10, 10));

 		getContentPane().add(new JLabel(Textometre.LABELS.getLocaliseString("tokenizer.class")));
 		getContentPane().add(listeTokenizer);

 		getContentPane().add(new JLabel(Textometre.LABELS.getLocaliseString("tokenizer.separators")));
 		getContentPane().add(strTokenizer);

 		getContentPane().add(new JLabel(""));
 		getContentPane().add(cmds);
		pack();
	}
	private void valid () {
		try{
			Class c = Class.forName((String)listeTokenizer.getSelectedItem());
			String str = strTokenizer.getText();
			str = str.replaceAll("\\\\t", "\t");
			str = str.replaceAll("\\\\n", "\n");
			str = str.replaceAll("\\\\r", "\r");
			preferencesController.setTokenizer(c);
			preferencesController.setStrTokenizer(str);
		} catch (Exception err) {
			System.err.println(err.getMessage());
		}
		dispose();
	}

	/**
	* Cette methode permet de lister toutes les classes d'un package donne
	*
	* @param pckgname Le nom du package a lister
	* @return La liste des classes
	*/
	public java.util.List<Class> getClasses(String pckgname) throws ClassNotFoundException, IOException {
		ArrayList<Class> classes = new ArrayList<Class>();

		// On recupere toutes les entrees du CLASSPATH
		String [] entries = System.getProperty("java.class.path").split(System.getProperty("path.separator"));

		// Pour toutes ces entrees, on verifie si elles contiennent un repertoire ou un jar
		for (int i = 0; i < entries.length; i++) {
			if(entries[i].endsWith(".jar")){
				classes.addAll(traitementJar(entries[i], pckgname));
			} else{
				classes.addAll(traitementRepertoire(entries[i], pckgname));
			}
		}

		return classes;
	}
	/**
	* Cette methode retourne la liste des classes presentes dans un jar du classpath et dans un package donne
	*
	* @param repertoire Le jar ou chercher les classes
	* @param pckgname Le nom du package
	* @return La liste des classes
	* @throws IOException
	* @throws ClassNotFoundException
	*/
	private Collection<Class> traitementJar(String jar, String pckgname) throws IOException, ClassNotFoundException {
		ArrayList<Class> classes = new ArrayList<Class>();

		JarFile jfile = new JarFile(jar);
		String pkgpath = pckgname.replace(".", "/");
		for (Enumeration entries = jfile.entries(); entries.hasMoreElements();) {
			JarEntry element = (JarEntry)entries.nextElement();

			// Si le nom de l'entree commence par le chemin du package et finit par .class
			if(element.getName().startsWith(pkgpath) && element.getName().endsWith(".class")){
				String nomFichier = element.getName().substring(pckgname.length() + 1);
				classes.add(Class.forName(pckgname + "." + nomFichier.split("\\.")[0]));
				//il faut enlever la classe d'interface Tokenizable.class qui fait planter
				if (nomFichier.equals("Tokenizable.class")) classes.remove(Class.forName(pckgname + "." + nomFichier.split("\\.")[0]));
			}
		}
		return classes;
	}
	/**
	* Cette methode retourne la liste des classes presentes dans un repertoire du classpath et dans un package donne.
	*
	* @param repertoire Le repertoire ou chercher les classes
	* @param pckgname Le nom du package
	* @return La liste des classes
	*/
	private Collection<Class> traitementRepertoire(String repertoire, String pckgname) throws ClassNotFoundException {
		ArrayList<Class> classes = new ArrayList<Class>();

		// On genere le chemin absolu du package
		StringBuffer sb = new StringBuffer(repertoire);
		String[] repsPkg = pckgname.split("\\.");
		for (int i = 0; i < repsPkg.length; i++) {
			sb.append(System.getProperty("file.separator") + repsPkg[i]);
		}
		File rep = new File(sb.toString());

		// Si le chemin existe, et que c'est un dossier, alors, on le liste
		if(rep.exists() && rep.isDirectory()){
			File[] liste = rep.listFiles();

			// Si le nom du fichier finit par .class
			for (int i = 0; i < liste.length; i++) {
				String ficname = liste[i].getName();
				if (ficname.endsWith(".class")) classes.add(Class.forName(pckgname + "." + ficname.split("\\.")[0]));
				//il faut enlever la classe d'interface Tokenizable.class qui fait planter
				if (ficname.equals("Tokenizable.class")) classes.remove(Class.forName(pckgname + "." + ficname.split("\\.")[0]));
			}
		}
		return classes;
	}
};