/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.preferences;

import socle.*;
import java.awt.*;
import javax.swing.*;
import java.util.Locale;
import java.text.Collator;
/*-----------------------------------------------------------------------*/

/** La classe <code>PreferencesView</code> definit une vue pour les preferences.
*
*/
public class PreferencesView extends JPanel implements PreferencesListener {

		private PreferencesController         controller;
		private Facets                        facets;

	/**
	 * Cree un panel dans lequel un lexique pourra s'afficher dans une table a deux colones ('forme graphique' et 'frequence').
	 **/
	public PreferencesView(PreferencesController controller) {
		this.controller = controller;
	}

	/**
	 * Fixe la police de caracteres utilisee pour l'affichage ainsi que le tag pour le hint et la couleur de fond pour les selections dans le texte.
	 **/
	public void displayOptions(String[] propertiesNames, Facets f) {
		this.facets = f;
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Font [] fonts = ge.getAllFonts();
		String[] fontNames = new String[fonts.length];
		for(int i=0; i<fonts.length; i++){
			fontNames[i]=fonts[i].getName();
		}
		JList fontlist = new JList(fontNames);
		fontlist.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		fontlist.setLayoutOrientation(JList.VERTICAL);
		fontlist.setVisibleRowCount(10);
		fontlist.setSelectedValue(controller.getFontName(), true);
		JScrollPane font_scroller = new JScrollPane(fontlist);
		JComponent panel_font = new JPanel(new BorderLayout());
		panel_font.add(font_scroller, BorderLayout.CENTER);

		JList taglist = new JList(facets.getNames());
		taglist.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		taglist.setLayoutOrientation(JList.VERTICAL);
		taglist.setVisibleRowCount(10);
		taglist.setSelectedValue(controller.getHint(), true);
		JScrollPane tag_scroller = new JScrollPane(taglist);
		JComponent panel_tag = new JPanel(new BorderLayout());
		panel_tag.add(tag_scroller, BorderLayout.CENTER);

		JColorChooser colorChooser = new JColorChooser(controller.getBgColor());

		ChooseObjectsPanel typeChooser = new ChooseObjectsPanel(propertiesNames, false, controller.getTypeDefinition());

		ChooseFacetsPanel facetsChooser = new ChooseFacetsPanel(propertiesNames, facets);




		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab(Textometre.LABELS.getLocaliseString("pref.bgColor.title"), null, colorChooser, Textometre.LABELS.getLocaliseString("pref.bgColor.message"));
		tabbedPane.addTab(Textometre.LABELS.getLocaliseString("pref.font.title"), null,    panel_font, Textometre.LABELS.getLocaliseString("pref.font.message"));
		tabbedPane.addTab(Textometre.LABELS.getLocaliseString("pref.hint.title"), null,    panel_tag, Textometre.LABELS.getLocaliseString("pref.hint.message"));
		tabbedPane.addTab(Textometre.LABELS.getLocaliseString("pref.type.title"), null,    typeChooser, Textometre.LABELS.getLocaliseString("pref.type.message"));
		tabbedPane.addTab(Textometre.LABELS.getLocaliseString("pref.facets.title"), null,  facetsChooser, Textometre.LABELS.getLocaliseString("pref.facets.message"));

		int answer = JOptionPane.showOptionDialog(null, tabbedPane, Textometre.LABELS.getLocaliseString("preferences.display"),
			JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,null, null, null);
		if (answer == JOptionPane.OK_OPTION) {
			String newHint = (String)taglist.getSelectedValue();
			if ((newHint!=null) && (!newHint.equals(controller.getHint()))) {
				controller.setHint((String)taglist.getSelectedValue());
			}

			String newFontName = (String)fontlist.getSelectedValue();
			if ((newFontName!=null) && (!newFontName.equals(controller.getFontName()))) {
				controller.setFontName((String)fontlist.getSelectedValue());
			}
			Color newColor = colorChooser.getColor();
			if ((newColor!=null) && (!newColor.equals(controller.getBgColor()))) {
				controller.setBgColor(newColor);
			}
			Object[] newTypeDefinition = typeChooser.getSelected();
			if (!(newTypeDefinition == controller.getTypeDefinition()) ) {
				controller.setTypeDefinition(newTypeDefinition);
			}
			Facets newFacets = facetsChooser.getNewValue();
			if (!newFacets.equals(facets)) {
				facets.setModel(newFacets);
			}
		}
	}
	public void chooseCollator() {
		Locale[] locales = Collator.getAvailableLocales();
		Locale loc = (Locale)JOptionPane.showInputDialog(null, Textometre.LABELS.getLocaliseString("win.collator.message"), Textometre.LABELS.getLocaliseString("win.collator.title"),
							JOptionPane.PLAIN_MESSAGE, null, locales, controller.getLocale4collator());
		if (loc != null) {
			controller.setLocale4collator(loc);
		}
	}
	public void chooseTokenizer() {
		DialogTokenizer dial = new DialogTokenizer(controller);
		dial.setVisible(true);
	}
	/**
	 * Capture des evenements de changement dans les preferences.
	 **/
	public void preferencesChanged(PreferencesChangedEvent event) {
	}
}