/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.reader;

import java.io.*;
import java.net.URL;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.Document;
import javax.swing.text.Element;

/*-----------------------------------------------------------------------*/

/** Lecteur pour le contenu textuel d'un fichier html.
*/
public class HtmDataReader implements ReaderAccessibility {
		private HTMLEditorKit       kit;
		private Document            doc;

	/**
	 * Cree un lecteur pour un fichier htm (version 3.2 uniquement).
	 * Ne recupere que le contenu textuel du document.
	 * @param fic Fichier html (version 3.2 uniquement) a lire
	 **/
	public HtmDataReader(File fic) throws Exception {
		kit = new HTMLEditorKit();
		doc = kit.createDefaultDocument();
		kit.read(new FileInputStream(fic), doc, 0);
	}

	/**
	 * Cree un lecteur pour une url htm (version 3.2 uniquement).
	 * Ne recupere que le contenu textuel du document.
	 * @param url URL du document html (version 3.2 uniquement) a lire
	 **/
	public HtmDataReader(URL url) throws Exception {
		kit = new HTMLEditorKit();
		doc = kit.createDefaultDocument();
		kit.read(url.openStream(), doc, 0);
	}

	/**
	 * Retourne la structure du document
	 **/
	public Element getStructure() {
		return doc.getDefaultRootElement();
	}
	/**
	 * Retourne la contenu textuel du document
	 **/
	public String getData() throws Exception {
		return doc.getText(0, doc.getLength());
	}
}
