/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.reader;

import java.io.*;
import java.net.*;
import java.nio.charset.Charset;
import java.util.regex.*;

import javax.swing.text.StyledEditorKit;
import javax.swing.text.Document;
import javax.swing.text.Element;

/*-----------------------------------------------------------------------*/

/** Lecteur pour d'un fichier texte.
*/
public class TextDataReader implements ReaderAccessibility {
		private Document         doc;
		private StyledEditorKit  kit;

	/**
	 * Cree un lecteur pour un fichier texte.
	 * @param fic Fichier texte a lire
	 * @param charset Encodage des carateres
	 **/
	public TextDataReader(File fic, Charset charset) throws Exception {
		kit = new StyledEditorKit();
		doc = kit.createDefaultDocument();
		kit.read(new InputStreamReader(new FileInputStream(fic), charset), doc, 0);
	}

	/**
	 * Cree un lecteur pour une url rtf.
	 * Ne recupere que le contenu textuel du document.
	 * @param url URL du document rtf a lire
	 **/
	public TextDataReader(URL url, String contentType) throws Exception {
		Charset charset = Charset.defaultCharset();
		Pattern p = Pattern.compile(".*charset[ \t]*=[ \t]*([^; \t]*)");
		Matcher m = p.matcher(contentType);
		if (m.matches()) {
			String encoding = m.group(1);
			try {
				charset = Charset.forName(encoding);
			} catch(Exception e) {
				System.err.println(e.getMessage());
			}
		}
		kit = new StyledEditorKit();
		doc = kit.createDefaultDocument();
		kit.read(new InputStreamReader(url.openStream(), charset), doc, 0);
	}

	/**
	 * Retourne la structure du document
	 **/
	public Element getStructure() {
		return doc.getDefaultRootElement();
	}
	/**
	 * Retourne la contenu textuel du document
	 **/
	public String getData() throws Exception {
		return doc.getText(0, doc.getLength());
	}
}
