/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.reader;

import java.util.ArrayList;
import javax.swing.text.Element;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;

import javax.swing.text.SimpleAttributeSet;
/*-----------------------------------------------------------------------*/

/**
 * Document modelisant ce qu'on retient pour l'edition d'un document XML.
 */
public class XMLDocument extends DefaultStyledDocument {

    /**
     * Liste d'ElementSpecs
     */
    private ArrayList<ElementSpec> list = null;

    public XMLDocument() {
        list = new ArrayList<ElementSpec>();
    }

    /**
     * Ajoute un element de contenu.
     */
    public void appendString(String str, AttributeSet attributes) {
		list.add(new ElementSpec(attributes, ElementSpec.StartTagType));
		char[] chars = str.toCharArray();
		list.add(new ElementSpec(attributes, ElementSpec.ContentType, chars, 0, chars.length));
		list.add(new ElementSpec(attributes, ElementSpec.EndTagType));
    }

    /**
     * Ajoute un element de debut de paragraphe.
     * Le nom de l'etiquette du paragraphe peut etre marquee dans un attribut tagname
     */
    public void appendStartTag(AttributeSet attributes) {
        list.add(new ElementSpec(attributes, ElementSpec.StartTagType));
    }
    /**
     * Ajoute un element de fin de paragraphe.
     * Le nom de l'etiquette du paragraphe peut etre marquee dans un attribut tagname
     */
    public void appendEndTag(AttributeSet attributes) {
        list.add(new ElementSpec(attributes, ElementSpec.EndTagType));
    }
    /**
     * Insere tous les elements ajoutes dans le document en une seule fois
     */
    public void processBatchUpdates(int offs) throws BadLocationException {
        ElementSpec[] inserts = new ElementSpec[list.size()];
        list.toArray(inserts);
        super.insert(offs, inserts);
    }
}
