/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.selection;

import java.util.*;
import java.awt.*;
import java.awt.datatransfer.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
/*-----------------------------------------------------------------------*/

/** Classe utilisee pour le transfert de donnees de type AbstractSelection vers un composant Swing.
*/
public class SelectionTransferHandler extends TransferHandler {
	DataFlavor localArrayListFlavor, serialArrayListFlavor;
	String localArrayListType = DataFlavor.javaJVMLocalObjectMimeType + ";class=java.util.ArrayList";
	JTree source = null;

	public SelectionTransferHandler() {
		try {
			localArrayListFlavor = new DataFlavor(localArrayListType);
		} catch (ClassNotFoundException e) {
			System.err.println("SelectionTransferHandler: unable to create data flavor");
		}
		serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
	}
	private boolean hasLocalArrayListFlavor(DataFlavor[] flavors) {
		if (localArrayListFlavor == null) {
			return false;
		}
		for (int i = 0; i < flavors.length; i++) {
			if (flavors[i].equals(localArrayListFlavor)) {
				return true;
			}
		}
		return false;
	}
	private boolean hasSerialArrayListFlavor(DataFlavor[] flavors) {
		if (serialArrayListFlavor == null) {
			return false;
		}
		for (int i = 0; i < flavors.length; i++) {
			if (flavors[i].equals(serialArrayListFlavor)) {
				return true;
			}
		}
		return false;
	}
	public boolean canImport(JComponent c, DataFlavor[] flavors) {
		if (hasLocalArrayListFlavor(flavors))  { return true; }
		if (hasSerialArrayListFlavor(flavors)) { return true; }
		return false;
	}
	protected Transferable createTransferable(JComponent c) {
		if (c instanceof JTree) {
			source = (JTree)c;
			TreePath[] nodes = source.getSelectionPaths();
			ArrayList<AbstractSelection> alist = new ArrayList<AbstractSelection>();
			boolean transferable = false;
			for (int i=0; i<nodes.length; i++) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodes[i].getLastPathComponent();
				if ((node != null) && (node.getUserObject() instanceof AbstractSelection)) {
					AbstractSelection selection = (AbstractSelection)node.getUserObject();
					alist.add(selection);
					transferable = true;
				}
			}
			if (transferable) {
				return new ArrayListTransferable(alist);
			} else {
				return null;
			}
		}
		return null;
	}
	public int getSourceActions(JComponent c) {
		return COPY;
	}

	/*********************************
	**
	**********************************/
	public class ArrayListTransferable implements Transferable {
		ArrayList data;

		public ArrayListTransferable(ArrayList alist) {
			data = alist;
		}
		public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
			if (!isDataFlavorSupported(flavor)) {
				throw new UnsupportedFlavorException(flavor);
			}
			return data;
		}
		public DataFlavor[] getTransferDataFlavors() {
			return new DataFlavor[] { localArrayListFlavor, serialArrayListFlavor };
		}
		public boolean isDataFlavorSupported(DataFlavor flavor) {
			if (localArrayListFlavor.equals(flavor)) {
				return true;
			}
			if (serialArrayListFlavor.equals(flavor)) {
				return true;
			}
			return false;
		}
	}
}
