/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.tokenizer;

import socle.trame.*;
import socle.lexicon.article.*;
import java.io.*;
import java.util.regex.*;

/*-----------------------------------------------------------------------*/

/** Segmenteur en mots utilisant une expression reguliere comme delimiteur
*/
public class RegexTokenizer extends Tokenizer {

		String regex;

	/**
	 * Construit un segmenteur qui decoupe des chaines de caracteres en fonction d'une expression reguliere.
	 *
	 * @param trame Trame textuelle
	 * @param str delimiteur sous forme d'une expression reguliere
	 **/
	public RegexTokenizer(Tokenizable trame, String str) {
		super(trame);
		this.regex = str;
	}

	/**
	 * Segmente une chaine de caracteres en items.
	 * La chaine lue est coupee dans un premier temps en lignes puis chaque fois que le l'expression reguliere est rencontree
	 * @param reader lecteur sur la chaine de caracteres
	 **/
	public void tokenize(BufferedReader reader) throws Exception {
		String line = reader.readLine();
		while (line != null) {
			Pattern pattern = Pattern.compile(regex);
			Matcher matcher = pattern.matcher(line);
			int start = 0;
			if (matcher.find()) {
				while (matcher.find()) {
					String separateur = matcher.group();
					int end = matcher.start();
					if (start < end) {
						String token = line.substring(start,end);
						newItem(token, Article.LEXICAL);
					}
					newItem(separateur, Article.SEPARATOR);
					start = matcher.end();
				}
			} else {
				if (!line.equals("")) newItem(line, Article.LEXICAL);
				start = line.length();
			}
			if (start < line.length()) {
				newItem(line.substring(start,line.length()-1), Article.LEXICAL);
			}
			line = reader.readLine();
			if (line!= null) newItem("\n", Article.NEWLINE);
		}
	}

}
