/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.tokenizer;

import socle.trame.*;
import socle.lexicon.article.*;
import java.io.*;
import java.util.Arrays;
/*-----------------------------------------------------------------------*/

/** Segmenteur en mots utilisant les une liste de caracteres separateurs
* passes en parametre.
*/
public class SeparatorTokenizer extends Tokenizer {

		String separators;
		char[] charArray;

	/**
	 * Construit un segmenteur qui decoupe des chaines de caracteres en fonction d'une liste de caracteres separateurs.
	 *
	 * @param trame Trame textuelle
	 * @param str Chaine de caracteres separateurs
	 **/
	public SeparatorTokenizer(Tokenizable trame, String str) {
		super(trame);
		this.separators = str;

		this.charArray = str.toCharArray();
		Arrays.sort(charArray);
	}

	/**
	 * Segmente une chaine de caracteres en items.
	 * La chaine lue est coupee a chaque fois qu'un separateur est rencontre
	 * @param reader lecteur sur la chaine de caracteres
	 **/
	public void tokenize(BufferedReader reader) throws Exception {
		char c;
		StringBuffer item = new StringBuffer();
		String line = reader.readLine();
		while (line != null) {
			for (int pos = 0; pos < line.length(); pos++) {
				c = line.charAt(pos);
				if (separators.indexOf(c) == -1) {
				//if (Arrays.binarySearch(charArray, c) == -1) {
					item.append(c);
				} else {
					if (item.length() > 0) {
						newItem(item.toString(), Article.LEXICAL);
						item = new StringBuffer();
					}
					newItem(Character.toString(c), Article.SEPARATOR);
				}
			}
			if (item.length() > 0) {
				newItem(item.toString(), Article.LEXICAL);
				item = new StringBuffer();
			}
			line = reader.readLine();
			if (line!= null) newItem("\n", Article.NEWLINE);
		}
	}

}
