/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JList;
/*-----------------------------------------------------------------------*/

/** Editeur pour une occurence de la trame.
*/
public class ListTrameOccView extends JList implements Selectable {

	private Object[]          content;
	private int               numocc;
	private boolean           select;
	private Color             default_color;
	private Color             default_bgcolor;
	private Color             current_bgcolor;

	/**
	 * Composant graphique de l'occurence.
	 * @param numocc      Numero de l'occurence.
	 **/
	public ListTrameOccView(int numocc, Component comp, Object[] cont) {
		super(cont);
		this.numocc = numocc;
		this.content = cont;
		this.select = false;
		default_color = getForeground();
		default_bgcolor = comp.getBackground();
		current_bgcolor = default_bgcolor;
		setFont(comp.getFont());
		setBackground(default_bgcolor);
		setOpaque(true);
		addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				clearSelection();
			}
			public void focusGained(FocusEvent e) {
				setSelectionInterval(0, getModel().getSize() -1);
			}
		});
	}

	/** Selectionne l'occurence.
	 **/
	public int getNumocc() {
		return numocc;
	}
	/** Selectionne l'occurence.
	 **/
	public void select(Color c) {
		setBackground(c);
		current_bgcolor = c;
		select = true;
	}

	/** Deselectionne l'occurence.
	 **/
	public void unselect() {
		setBackground(default_bgcolor);
		current_bgcolor = default_bgcolor;
		select = false;
	}

	/** Donne le focus a l'occurence.
	 **/
	public void focus() {
		setBackground(Color.GREEN);
	}
	public void unfocus() {
		setBackground(current_bgcolor);
	}
}