/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.preferences.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JTextPane;
import javax.swing.text.Style;
import javax.swing.text.StyledDocument;
import javax.swing.text.StyleConstants;
/*-----------------------------------------------------------------------*/

/** Editeur pour une occurence de la trame.
*/
public class StringTrameOccView implements Selectable {

	private String            content;
	private int               numocc;
	private boolean           select;
	private Color             default_bgcolor;
	private Color             current_bgcolor;
	private StyledDocument    sd;
	private Style             style;
	private int               startPos;

	/**
	 * Composant graphique de l'occurence.
	 * @param numocc      Numero de l'occurence.
	 **/
	public StringTrameOccView(int numocc, JTextPane editor, String content) {
		//super(content);
		this.numocc = numocc;
		this.content = content;
		this.select = false;
		this.sd = (StyledDocument)editor.getDocument();
		this.style = editor.getStyle(PreferencesModel.STYLE_HIGHTLIGHT);
		this.startPos = sd.getLength();

		default_bgcolor = editor.getBackground();
		current_bgcolor = default_bgcolor;

		/*addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				selectAll();
			}
		});*/
	}

	/** Retourne le numero de l'occurence.
	 **/
	public int getNumocc() {
		return numocc;
	}
	/** Donne le focus a l'occurence.
	 **/
	public void focus() {
		StyleConstants.setBackground(style, Color.GREEN);
		sd.setCharacterAttributes(startPos, content.length(), style, false);
	}
	public void unfocus() {
		StyleConstants.setBackground(style, current_bgcolor);
		sd.setCharacterAttributes(startPos, content.length(), style, false);
	}
	/** Selectionne l'occurence.
	 **/
	public void select(Color c) {
		StyleConstants.setBackground(style, c);
		sd.setCharacterAttributes(startPos, content.length(), style, false);
		current_bgcolor = c;
		select = true;
	}

	/** Deselectionne l'occurence.
	 **/
	public void unselect() {
		StyleConstants.setBackground(style, default_bgcolor);
		sd.setCharacterAttributes(startPos, content.length(), style, false);
		current_bgcolor = default_bgcolor;
		select = false;
	}

	public String toString() {
		return content;
	}

}