/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2010 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import java.awt.*;
import java.awt.event.*;
import javax.swing.SwingUtilities;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
/*-----------------------------------------------------------------------*/

/** Editeur pour une occurence de la trame.
*/
public class TextFieldTrameOccView extends JTextField implements Selectable {

	private String            content;
	private int               numocc;
	private boolean           select;
	private Color             default_bgcolor;
	private Color             current_bgcolor;

	/**
	 * Composant graphique de l'occurence.
	 * @param numocc      Numero de l'occurence.
	 **/
	public TextFieldTrameOccView(int numocc, Component comp, String content, String hint) {
		super(content);
		this.numocc = numocc;
		this.content = content;
		this.select = false;

		setOpaque(true);
		default_bgcolor = comp.getBackground();
		Font font = comp.getFont();
		setFont(font);
		setBackground(default_bgcolor);
		current_bgcolor = default_bgcolor;
		setEditable(false);
		setBorder(new EmptyBorder(0,0,0,0));
		FontMetrics fontMetrics = getFontMetrics(font);
		int textWidth = SwingUtilities.computeStringWidth(fontMetrics, content);
		setPreferredSize(new Dimension(textWidth, (int)getPreferredSize().getHeight()));
		setMaximumSize(new Dimension(textWidth, (int)getPreferredSize().getHeight()));
		setToolTipText(hint);
		addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				selectAll();
			}
		});
	}

	/** Retourne le numero de l'occurence.
	 **/
	public int getNumocc() {
		return numocc;
	}
	/** Donne le focus a l'occurence.
	 **/
	public void focus() {
		setBackground(Color.GREEN);
	}
	public void unfocus() {
		setBackground(current_bgcolor);
	}
	/** Selectionne l'occurence.
	 **/
	public void select(Color c) {
		setBackground(c);
		current_bgcolor = c;
		select = true;
	}

	/** Deselectionne l'occurence.
	 **/
	public void unselect() {
		setBackground(default_bgcolor);
		current_bgcolor = default_bgcolor;
		select = false;
	}

}