/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.*;

import socle.util.*;
import java.util.*;

/*-----------------------------------------------------------------------*/

/** Modele de donnees definissant une trame numerique accompagnee d'un lexique.
*/
public class Tnum extends ArrayList<Object> {
		//clef   = numero de premiere apparition d'une unite
		//valeur = frequence d'unite de ce type dans la trame
		private HashMap<Integer, Integer>               mapFreq;

		//clef   = une forme
		//valeur = numero de la premiere unite qui porte cette forme
		private HashMap<Object, Integer>               properties;

		//clef   = numero de la premiere unite qui porte cette forme
		//valeur = la forme de l'unite
		private HashMap<Integer, Object>               invert;

		//donne le prochain numero libre
		int compteur;

		public static String GRAPH  = "graph";
		public static String STATUS = "status";


	/**
	 * Construit une nouvelle trame numerisee.
	 **/
	public Tnum() {
		super();
		this.mapFreq    = new HashMap<Integer, Integer>();
		this.properties = new HashMap<Object, Integer>();
		this.invert     = new HashMap<Integer, Object>();
		this.compteur   = 0;
	}

	/**
	 * Construit une nouvelle trame numerisee.
	 **/
	public Tnum(int initialCapacity) {
		super(initialCapacity);
		this.mapFreq    = new HashMap<Integer, Integer>();
		this.properties = new HashMap<Object, Integer>();
		this.invert     = new HashMap<Integer, Object>();
		this.compteur   = 0;
	}

	/**
	 * Construit une nouvelle trame numerisee avec les elements de la Collection specifiee.
	 **/
	public Tnum(Collection<Object> c) {
		this();
		addAll(c);
	}

	/**
	 * Ajoute l'element specifie a la fin de la liste.
	 **/
	public boolean add(Object o) {
		Integer idnum = setNumID(o);
		boolean ok = super.add(idnum);
		if (ok) {
			add2map(idnum);
		}
		return ok;
	}
	/**
	 * Ajoute l'element specifie a la fin de la liste sans alimenter les dictionnaires.
	 **/
	public boolean superadd(Integer i) {
		return super.add(i);
	}

	/**
	 * Ajoute l'element specifie a la position specifiee dans la liste.
	 **/
	public void add(int index, Object o) {
		Integer idnum = setNumID(o);
		super.add(index, idnum);
		add2map(idnum);
	}

	/**
	 * Ajoute a la fin de la liste tous les elements de la Collection specifiee.
	 **/
	public boolean addAll(Collection c) {
		Iterator iterator = c.iterator();
		while (iterator.hasNext()) {
			Object o = iterator.next();
			if (!add(o)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Insere tous les elements de la Collection specifiee dans la liste, en partant de la position specifiee.
	 **/
	public boolean addAll(int index, Collection c) {
		Iterator iterator = c.iterator();
		int i = 0;
		while (iterator.hasNext()) {
			Object o = iterator.next();
			add(index+i, o);
			i++;
		}
		return true;
	}

	/**
	 * Supprime tous les elements de la liste.
	 **/
	public void clear() {
		super.clear();
		clearMap();
	}

	/**
	 * Supprime l'element specifie dans la liste.
	 **/
	public Integer remove(int index) {
		Integer o = (Integer)super.remove(index);
		remove2map((Integer)o);
		return o;
	}

	/**
	 * Remplace l'element a la position specifiee dans la liste par l'element specifie.
	 * Attention la fonction ne modifie pas les lexiques.
	 **/
	//public Object set(int index, Object idnum) {
	public Integer set(int index, Object o) {
		Integer idnum = setNumID(o);
		remove2map((Integer)get(index));
		add2map(idnum);
		return (Integer)super.set(index, idnum);
	}

/******************************************
** Fonctions ajoutees a la classe ArrayList
*******************************************/

	/**
	 * Calcule le numero d'une occurrence et l'ajoute aux lexiques (properties et invert).
	 **/
	private Integer setNumID(Object occurrence) {
		Integer idnum = getNumap(occurrence);
		if (idnum == null) {
			idnum = new Integer(compteur);
			properties.put(occurrence, idnum);
			invert.put(idnum, occurrence);
			compteur++;
		}
		return idnum;
	}

	/**
	 * Increment le numero dans le lexique (mapFreq) ou le cree numero si il n'y est pas encore.
	 **/
	private void add2map(Integer idnum) {
		if (mapFreq.containsKey(idnum)) {
			int before = mapFreq.get(idnum).intValue();
			mapFreq.put(idnum, new Integer(before+1));
		} else {
			mapFreq.put(idnum, new Integer(1));
		}
	}

	/**
	 * Diminue la frequence d'un point la frequence du numero (mapFreq) ou le numero lui-meme si c'est le dernier (invert et properties).
	 **/
	private void remove2map(Integer idnum) {
		if (mapFreq.containsKey(idnum)) {
			int before = mapFreq.get(idnum).intValue();
			if (before > 1) {
				mapFreq.put(idnum, new Integer(before-1));
			} else {
				mapFreq.remove(idnum);
				Object o = invert.get(idnum);
				invert.remove(idnum);
				properties.remove(o);
			}
		}
	}

	/**
	 * Fixe la valeur d'un numero dans les tables (invert et properties);
	 **/
	private void set2map(Integer idnum, Object o) {
		if (!invert.get(idnum).equals(o)) {
			properties.put(o, idnum);
			invert.put(idnum, o);
		}
	}

	/**
	 * Change la forme d'une occurence.
	 **/
	public void changeOcc(int numocc, String new_str) {
		Integer new_num = setNumID(new_str);
		Integer old_num = (Integer)get(numocc);
		if (!new_num.equals(old_num)) {
			add2map(new_num);
			remove2map(old_num);
			super.set(numocc, new_num);
		}
		set2map(new_num, new_str);
	}

	/**
	 * Vide les tables.
	 **/
	private void clearMap() {
		mapFreq.clear();
		properties.clear();
		invert.clear();
	}

	/**
	 * Retourne le numero de premiere apparition de l'occurence.
	 **/
	public int getInt(int numocc) {
		return ((Integer)get(numocc)).intValue();
	}

	/**
	 * Retourne la forme de l'occurence.
	 **/
	public String getString(int numocc) {
		Integer num = (Integer)get(numocc);
		return getGraph(num);
	}

	/**
	 * Retourne la forme du tnum.
	 **/
	public String getString() {
		StringBuffer content = new StringBuffer(100000);
		for (int i=0; i<size(); i++) {
			content.append(getString(i));
		}
		return content.toString();
	}
	/**
	 * Retourne la forme d'une portion du tnum.
	 **/
	public String getString(Range range) {
		StringBuffer content = new StringBuffer(100000);
		for (int i=range.getMinValue(); i<=range.getMaxValue(); i++) {
			content.append(getString(i));
		}
		return content.toString();
	}

	/**
	 * Retourne la liste de toutes les formes du dictionnaire (properties).
	 **/
	public Set getAllForms() {
		return properties.keySet();
	}

	/**
	 * Retourne le numero d'apparition de l'occurence (properties).
	 **/
	public Integer getNumap(Object occurrence) {
		return properties.get(occurrence);
	}

	/**
	 * Retourne la forme graphique correspondant a un numero d'apparition (invert).
	 **/
	public String getGraph(Integer numap) {
		if (numap != null) {
			return invert.get(numap).toString();
		} else {
			return "";
		}
	}
	public int getFreq(Integer numap) {
		if (numap != null) {
			return mapFreq.get(numap).intValue();
		} else {
			return 0;
		}
	}
	public void copyLexicon(Tnum tnum) {
		this.mapFreq    = new HashMap<Integer, Integer>(tnum.mapFreq);
		this.properties = new HashMap<Object, Integer>(tnum.properties);
		this.invert     = new HashMap<Integer, Object>(tnum.invert);
		this.compteur   = tnum.compteur;
	}
	/**
	 * Calcule le tnum correspondant a la composition du tnum avec celui specifie.
	 **/
	public Tnum croise(Tnum tnum) {
		if (tnum.size() != size()) {
			return null;
		}
		Tnum result = new Tnum();
		for (int i=0; i<size(); i++) {
			NamedPoint point = new NamedPoint(tnum.getInt(i), getInt(i), getGraph((Integer)get(i))+tnum.getGraph((Integer)tnum.get(i)));
			result.add(point);
		}
		return result;
	}
	/**
	 * Calcule le tnum correspondant a la composition du tnum avec une chaine de caractere.
	 **/
	public Tnum croise(String litteral) {
		Tnum result = new Tnum();
		for (int i=0; i<size(); i++) {
			result.add(getGraph((Integer)get(i))+litteral);
		}
		return result;
	}
	public void voir() {
		System.out.println(mapFreq);
	}
	public String getDico() {
		StringBuffer sb = new StringBuffer();
		Set<Integer> ensemble = invert.keySet();
		Iterator<Integer> iterator = ensemble.iterator();
		while (iterator.hasNext()) {
			Integer num = iterator.next();
			String val = getGraph(num);
			int freq = getFreq(num);
			sb.append("<entry><id>");
			sb.append(num);
			sb.append("</id><freq>");
			sb.append(freq);
			sb.append("</freq><f>");
			sb.append(XMLString.toString(val));
			sb.append("</f></entry>\n");
		}
		return sb.toString();
	}
	public String getData() {
		StringBuffer sb = new StringBuffer();
		sb.append("<data>");
		for (int i=0; i<size(); i++) {
			sb.append(getInt(i));
			sb.append("\t");
		}
		sb.append("</data>\n");
		return sb.toString();
	}
	/**
	 * Force le contenu.
	 **/
	public void setData(String s) {
		String[] result = s.split("\\t");
		ArrayList<Integer> liste = new ArrayList<Integer>();
		for (int i=0; i<result.length; i++) {
			liste.add(new Integer(result[i]));
		}
		super.addAll(liste);
	}
	/**
	 * Force le contenu.
	 **/
	public void setDico(HashMap<Integer, Object> tab_invert, HashMap<Integer, Integer> tab_freq, HashMap<Object, Integer> tab_properties) {
		mapFreq = tab_freq;
		invert = tab_invert;
		properties = tab_properties;
		compteur = invert.size();
	}
}