/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.*;
import socle.util.*;
import socle.cadre.*;
import socle.selection.*;
import socle.preferences.*;
import socle.lexicon.*;
import socle.lexicon.article.*;

import java.util.*;
import java.awt.Container;
import javax.swing.*;

import javax.swing.event.*;
import javax.swing.undo.*;
/*-----------------------------------------------------------------------*/

/** Controlleur d'une trame textuelle.
*/
public class TrameController {
	private TrameView         graphView = null;
	private TrameModel        model = null;

	/**
	 * Creation du controleur.
	 * @param model                Modele de donnees de la trame lexicale.
	 **/
	public TrameController (TrameModel model) {
		this.model = model;
		graphView = new TrameView(this);
		addTrameListener(graphView);
		model.getFacets().addFacetsListener(graphView);
	}
	public void addTrameListener(TrameListener listener) {
		model.addTrameListener(listener);
	}
	public void addPreferencesListener(PreferencesModel preferencesModel) {
		preferencesModel.addPreferencesListener(graphView);
	}
	public void addLexListener(LexModel lexicon) {
		lexicon.addLexListener(graphView);
	}
	public void addCadreListener(CadreModel cadre) {
		cadre.addCadreListener(graphView);
	}
	public void displayView(Container container) {
		container.add(graphView);
	}
	public ZoneTrame toZoneTrame(ZoneText zone) {
		return model.toZoneTrame(zone);
	}
	public ZoneText toZoneText(ZoneTrame zone) {
		return model.toZoneText(zone);
	}

	public int getTrameSize() {
		return model.size();
	}
	/**
	 * Retourne l'occurrence  la position numocc.
	 **/
	public OccurrenceModel getOccurrence(int numocc) throws Exception {
		return model.getOccurrence(numocc);
	}
	/**
	 * Retourne le tnum du nom specifie.
	 **/
	public Tnum getTnum(String name) {
		return model.getTnum(name);
	}
	/**
	 * Fixe le tnum du nom specifie.
	 **/
	public void setTnum(String name, Tnum atnum) {
		model.setTnum(name, atnum);
	}
	/**
	 * Retourne la trame correspondant a la zone.
	 **/
	public TrameModel subtrame(ZoneTrame zone) {
		return model.subtrame(zone);
	}

	/**
	 * Retourne une selection de toutes les occurrence de l'objet dans la trame.
	 **/
	public AbstractSelection getOccs(Object obj) {
		return model.getOccs(obj);
	}
	/**
	 * Retourne une selection de toutes les occurrence de l'objet dans une zone specifiee de la trame.
	 **/
	public AbstractSelection getOccs(Object obj, ZoneTrame zt) {
		return model.getOccs(obj, zt);
	}
	/**
	 * Retourne une selection de toutes les segments selectionnes dans l'editeur de texte.
	 **/
	public AbstractSelection getSelectionSelected() {
		return graphView.getSelectionSelected();
	}
	public void find() {
		graphView.find();
	}

	/**
	 * Affiche une fenetre modale pour editer les proprietes de l'occurence
	 * @return Une nouvelle copie modifiee de l'occurence.
	 **/
	public OccurrenceModel editOccurrence(OccurrenceModel old_occ) {
		OccurrenceModel new_occ = new OccurrenceModel(old_occ);
		OccurrenceController occurrenceController = new OccurrenceController(new_occ);
		if (occurrenceController.getModif()) {
			return new_occ;
		} else {
			return null;
		}
	}
	/**
	 * Affiche une fenetre modale pour editer les proprietes d'une occurrence resultant de la concatenation d'une suite d'occurrences
	 * @return Une nouvelle copie modifiee de l'occurence.
	 **/
	public OccurrenceModel editTrame(ArrayList old_occs, int numocc) {
		//attention au numapp = 0
		HashMap<String,String> properties = new HashMap<String,String>();
		for (int i=0; i<old_occs.size(); i++) {
			OccurrenceModel occ = (OccurrenceModel)old_occs.get(i);
			String[] props = getPropertyNames();
			for (int n=0; n<props.length; n++) {
				String propertieName = props[i];
				String propertieValue = (occ.getValue(propertieName) == null) ? "": occ.getValue(propertieName);
				String old_propertieValue = (properties.get(propertieName) == null) ? "": (String)properties.get(propertieName);
				properties.put(propertieName, old_propertieValue+propertieValue);
			}
		}

		OccurrenceModel new_occ = new OccurrenceModel((String)properties.get(Tnum.GRAPH), 0, numocc, properties);
		OccurrenceController occurrenceController = new OccurrenceController(new_occ);
		if (occurrenceController.getModif()) {
			return new_occ;
		} else {
			return null;
		}
	}


	/**
	 * Remplace une occurrence par une autre occurrence (declenche la mise a jour du lexique).
	 **/
	public void replaceOccurence(OccurrenceModel old_occ, OccurrenceModel new_occ) {
		try {
			//mise a jour de la vue si besoin
			int numocc = old_occ.getNumocc();
			String new_str = new_occ.getValue(model.getCurrentTrameName());
			String old_str = old_occ.getValue(model.getCurrentTrameName());
			if (!new_str.equals(old_str)) {
				int start = model.getStartPosition(numocc);
				graphView.changeContent(start, old_str.length(), new_str);
			}

			//mise a jour du modele
			model.changeOccurence(old_occ, new_occ);
			model.fireLexChanged();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Remplace une suite d'occurrence par une occurrence.
	 **/
	public void replaceOccByOccs(OccurrenceModel old_occ, ArrayList<OccurrenceModel> new_occs, int numocc) {
		try {
			//mise a jour de la vue
			int insertPt = model.getStartPosition(numocc);
			String str ="";
			for (int i=0; i<new_occs.size(); i++) {
				OccurrenceModel occ = new_occs.get(i);
				str += occ.getValue(model.getCurrentTrameName());
			}
			graphView.changeContent(insertPt, old_occ.getValue(model.getCurrentTrameName()).length(), str);

			//mise a jour du modele
			model.replaceOccByOccs(old_occ, new_occs, numocc);
			model.fireLexChanged();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Remplace une suite d'occurrence par une occurrence.
	 **/
	public void replaceOccsByOcc(ArrayList<OccurrenceModel> old_occs, OccurrenceModel new_occ, int numocc) {
		try {
			//mise a jour de la vue
			int insertPt = model.getStartPosition(numocc);
			int old_lenght = 0;
			for (int i=0; i<old_occs.size(); i++) {
				OccurrenceModel occ = old_occs.get(i);
				old_lenght += occ.getValue(model.getCurrentTrameName()).length();
			}
			graphView.changeContent(insertPt, old_lenght, new_occ.getValue(model.getCurrentTrameName()));

			//mise a jour du modele
			model.replaceOccsByOcc(old_occs, new_occ, numocc);
			model.fireLexChanged();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Retourne les facettes;.
	 **/
	public Facets getFacets() {
		return model.getFacets();
	}
	/**
	 * Retourne la definition d'une facette;.
	 **/
	public Object[] getFacetDefinition(String facet) {
		return model.getFacets().getFacetValue(facet);
	}
	/**
	 * Retourne l'ensemble des noms des proprietes des items.
	 **/
	public String[] getPropertyNames() {
		return model.getPropertyNames();
	}

	/**
	 * Affiche le tnum du meme nom.
	 **/
	public void showTnum(String tnumName, boolean changeLex, boolean changeTrame) {
		if (changeLex) {
			model.setLexModel(tnumName);
		}
		if (changeTrame) {
			model.setCurrentTnum(tnumName);
			model.fireTrameChanged();
		}
	}

	/**
	 * Retourne le nom de la vue du lexique courant.
	 **/
	public String getCurrentLexName() {
		return model.getCurrentLexName();
	}

	/**
	 * Notifie un changement intervenu dans le lexique au travers de sa vue.
	 **/
	public void notifyTrameChanged() {
		model.fireLexChanged();
		model.fireTrameChanged();
	}

	public void changeAllOccurences(AbstractSelection sel, String new_graph, String currentTnumName) {
		model.changeAllOccurences(sel, new_graph, currentTnumName);
	}
}
