/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.*;
import socle.util.*;
import socle.cadre.*;
import socle.selection.*;
import socle.preferences.*;
import socle.lexicon.*;
import socle.lexicon.article.*;

import java.util.EventObject;
import java.util.Iterator;
import java.util.ArrayList;
import java.text.NumberFormat;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.undo.*;
import javax.swing.event.*;
/*-----------------------------------------------------------------------*/

/** Editeur pour une trame textuelle.
*/
public class TrameView extends JPanel implements TrameListener, PreferencesListener, CadreListener, LexListener, FacetsListener {

		// Les segments selectionnes dans l'editeur
		private IntervalSelection         selectionSelected;
		private ZoneTrame                 focusZone;
		// L'editeur de texte
		private JTextPane                 editor;
		private StyledDocument            sd;
		private Style                     hightlightStyle;
		private TrameController           trameController;
		private Color                     selection_color;
		private String                    hintName;
		private int                       limit;
		private int                       from;
		private int                       style;
		private JSlider                   from_slider;
		private ArrayList<Selectable>     listOfComponents;
		private JLabel                    facetsLabel, themeLabel, nbOfItemsLabel;
		private JComboBox                 facetsComboBox;


		public static int                 LIMIT   = 1000;

	/**
	 * Editeur de texte qui affiche le contenu typographique d'une trame.
	 * @param control      Controller de l'editeur de trame.
	 **/
	public TrameView(TrameController control){
		setLayout(new BorderLayout());
		//setPreferredSize(new Dimension(100, 40));
		this.trameController = control;
		this.hintName = PreferencesModel.HINT;;
		this.limit = LIMIT;
		this.from = 0;
		this.style = 0;
		this.selection_color = Color.RED;
		this.selectionSelected = new IntervalSelection("selected", trameController.getTrameSize());
		focusZone = new ZoneTrame(-1,-1);
		this.listOfComponents = new ArrayList<Selectable>();

		//boite de dialogue pour fixer l'item de depart de l'affichage
		from_slider = new JSlider(0, trameController.getTrameSize() / limit, 0);
		//from_slider.setMajorTickSpacing(trameController.getTrameSize() / (limit*25));
		//from_slider.setPaintTicks(true);
		from_slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
				if (!source.getValueIsAdjusting()) {
					from = (int)source.getValue()*limit;
					int to = from + limit;
					if (to > trameController.getTrameSize()) to = trameController.getTrameSize();
					source.setToolTipText("["+from+"-"+to+"]");
					maj();

				}
			}
		});

		//boite de dialogue pour fixer le nombre d'item a afficher
		JComboBox limit_cb = new JComboBox(new Integer[] {2000, 1000, 500, 100});
		limit_cb.setSelectedItem(new Integer(limit));
		limit_cb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox jcb = (JComboBox)e.getSource();
				Integer i = (Integer)jcb.getSelectedItem();
				int val = from_slider.getValue() * limit;
				limit = i.intValue();
				from_slider.setMaximum(trameController.getTrameSize() / (limit*25));
				from_slider.setValue(val / limit);
				//from_slider.setMajorTickSpacing(trameController.getTrameSize() / 25);
				maj();
			}
		});



		//menu pour choisir le type d'affichage du texte
		String[] fieldValues = {"normal", "avec hint", "interlineaire"};
		final JComboBox styleComboBox = new JComboBox(fieldValues);
		styleComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selectedIndex = styleComboBox.getSelectedIndex();
				if ((selectedIndex != style) && (selectedIndex != -1)) {
					style = selectedIndex;
					maj();
				}
			}
		});


		facetsComboBox = new JComboBox();
		facetsComboBox.addItemListener(new myItemListener(PreferencesModel.DEFAULT_FACET));

		this.facetsLabel = new JLabel(Textometre.LABELS.getLocaliseString("field.names"));
		this.themeLabel = new JLabel(Textometre.LABELS.getLocaliseString("term.theme"));
		this.nbOfItemsLabel = new JLabel(Textometre.LABELS.getLocaliseString("wintrame.nb_of_items"));

		JPanel cmds = new JPanel();
		cmds.add(facetsLabel);
		cmds.add(facetsComboBox);
		cmds.add(from_slider);
		cmds.add(nbOfItemsLabel);
		cmds.add(limit_cb);
		cmds.add(themeLabel);
		cmds.add(styleComboBox);

		add(cmds, BorderLayout.NORTH);

		this.editor = new JTextPane();
		editor.setBackground(Color.WHITE);
		this.sd = (StyledDocument)editor.getDocument();

		DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
			public synchronized void drop (DropTargetDropEvent dropTargetDropEvent) {
				try {
					Transferable tr = dropTargetDropEvent.getTransferable();
					DataFlavor serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
					if (tr.isDataFlavorSupported (serialArrayListFlavor)) {
						dropTargetDropEvent.acceptDrop (DnDConstants.ACTION_COPY);
						ArrayList list = (ArrayList)tr.getTransferData(serialArrayListFlavor);
						AbstractSelection selectionAll = getSel(list);
						add2Selection(selectionAll);
						dropTargetDropEvent.getDropTargetContext().dropComplete(true);
				  } else {
					System.err.println("Rejected");
					dropTargetDropEvent.rejectDrop();
				  }
				} catch (UnsupportedFlavorException e) {
					e.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				} catch (Exception err) {
					err.printStackTrace();
					dropTargetDropEvent.rejectDrop();
				}
			}
		};

		DropTarget dropTarget = new DropTarget(editor, dropTargetAdapter);
		editor.setDragEnabled(true);
		StringTransferHandler arrayListHandler = new StringTransferHandler();
		editor.setTransferHandler(arrayListHandler);

		editor.setEditable(false);

		Style defautStyle = editor.getStyle(StyleContext.DEFAULT_STYLE);
		this.hightlightStyle = editor.addStyle(PreferencesModel.STYLE_HIGHTLIGHT, defautStyle);
		JScrollPane scroller = new JScrollPane(editor, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		add(scroller, BorderLayout.CENTER);
	}

	/**
	 * Affiche le tnum du meme nom.
	 **/
	private void showTnum(String name, boolean changeLex, boolean changeTrame) {
		trameController.showTnum(name, changeLex, changeTrame);
	}

	/**
	 * Recherche d'une chaine de caractere dans le texte de la trame.
	 **/
	public void find() {
		Find dial = new Find(editor);
		dial.setVisible(true);
	}
	/**
	 * Donne le focus a la zone de texte.
	 **/
	private void focus(ZoneTrame zoneTrame) {
		for (int i=0; i<listOfComponents.size(); i++){
			Selectable comp = listOfComponents.get(i);
			if (focusZone.contains(comp.getNumocc())) {
				comp.unfocus();
			}
			if (zoneTrame.contains(comp.getNumocc())) {
				comp.focus();
			}
		}
		focusZone = zoneTrame;
	}
	/**
	 * Selectionne la zone de texte.
	 **/
	private void select(ZoneTrame zoneTrame) {
		for (int i=0; i<listOfComponents.size(); i++){
			Selectable comp = listOfComponents.get(i);
			if (zoneTrame.contains(comp.getNumocc())) {
				comp.select(selection_color);
			}
		}
	}
	/**
	 * Change le contenu d'une zone de texte.
	 **/
	public void changeContent(int offset, int length, String s) {
		try {
			sd.remove(offset, length);
			sd.insertString(offset, s, new SimpleAttributeSet());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Ajoute les zones dans la liste des zones selectionnees de la trame.
	 * @param selection     selection d'unites dans la trame
	 **/
	private void add2Selection(AbstractSelection selection) {
		ArrayList tableau = selection.toArray();
		for (int i=0; i<tableau.size(); i++) {
			Range range = (Range)tableau.get(i);
			ZoneTrame zone = new ZoneTrame(range.getMinValue(), range.getMaxValue());
			select(zone);
		}
	}
	/**
	 * Supprime tous les surlignages dans le texte.
	 **/
	private void clearBgColor() {
		sd.setCharacterAttributes(0, sd.getLength(), editor.getStyle(PreferencesModel.STYLE_UNHIGHTLIGHT), false);
		selectionSelected = new IntervalSelection("selected", trameController.getTrameSize());
	}

	/**
	 * Retourne la liste de toutes les zones selectionnees.
	 **/
	public AbstractSelection getSelectionSelected() {
		return selectionSelected;
	}

	/**
	 * Retourne la selection correspondant a l'ensemble des items de la liste.
	 **/
	private AbstractSelection getSel(ArrayList list) {
		return getSel(list, new ZoneTrame(0, trameController.getTrameSize()-1));
	}
	private AbstractSelection getSel(ArrayList list, ZoneTrame zt) {
		AbstractSelection selectionAll = new BooleanSelection(trameController.getTrameSize());
		for (int i=0; i<list.size(); i++) {
			Object o = list.get(i);
			AbstractSelection tmp = trameController.getOccs(o, zt);
			selectionAll = selectionAll.union(tmp);
		}
		return selectionAll;
	}

	/**
	 * Capture les evenements en provenance des preferences (PreferencesListener).
	 **/
	public void preferencesChanged(PreferencesChangedEvent event) {
		if(event instanceof PreferencesChangedFontEvent) {
			PreferencesChangedFontEvent e = (PreferencesChangedFontEvent)event;
			String name = e.getNewFontName();
			Font font = editor.getFont();
			editor.setFont(new Font(name, font.getStyle(), font.getSize()));
			editor.updateUI();
			maj();
		} else if(event instanceof PreferencesChangedBgColorEvent) {
			PreferencesChangedBgColorEvent e = (PreferencesChangedBgColorEvent)event;
			selection_color = e.getNewBgColor();
			StyleConstants.setBackground(editor.getStyle(PreferencesModel.STYLE_HIGHTLIGHT), selection_color);
		} else if(event instanceof PreferencesChangedHintEvent) {
			PreferencesChangedHintEvent e = (PreferencesChangedHintEvent)event;
			hintName = e.getNewHintName();
			maj();
		} else if (event instanceof PreferencesChangedLocaleEvent) {
			this.facetsLabel.setText(Textometre.LABELS.getLocaliseString("field.names"));
			this.themeLabel.setText(Textometre.LABELS.getLocaliseString("term.theme"));
			this.nbOfItemsLabel.setText(Textometre.LABELS.getLocaliseString("wintrame.nb_of_items"));
		}
	}
	/**
	 * Capture les evenements en provenance de la trame (TrameListener).
	 **/
	public void trameChanged(TrameEvent event) {
		if (event instanceof TrameFieldsChangedEvent) {
			TrameFieldsChangedEvent e = (TrameFieldsChangedEvent)event;
			maj();
		} else if (event instanceof TrameChangedEvent) {
			from_slider.setMaximum(trameController.getTrameSize() / limit);
			maj();
		}
	}
	private String getFace(OccurrenceModel occ, Object[] facetDefinition) {
		String result = "";
		for (int n=0; n<facetDefinition.length; n++) {
			Object o = facetDefinition[n];
			if (o instanceof Litteral) {
				result += o.toString();
			} else {
				result += occ.getValue(o.toString());
			}
		}
		return result;
	}
	private ArrayList<String> getFaces(OccurrenceModel occ) {
		ArrayList<String> result = new ArrayList<String>();
		for (int i=0; i<facetsComboBox.getItemCount(); i++) {
			String face = (String)facetsComboBox.getItemAt(i);
			Object[] definition = trameController.getFacetDefinition(face);
			result.add(getFace(occ, definition));
		}
		return result;
	}
	private void maj() {
		String face_selected = (String)facetsComboBox.getSelectedItem();
		if (face_selected == null) face_selected = PreferencesModel.DEFAULT_FACET;
		Object[] definition = trameController.getFacetDefinition(face_selected);
		try {
			listOfComponents.clear();
			sd.remove(0, sd.getLength());
			int max = trameController.getTrameSize();
			int until = from + limit;
			if (until > max) until = max;
			for (int i=from; i<until; i++) {
				OccurrenceModel occ = trameController.getOccurrence(i);
				if (occ.getValue(Tnum.GRAPH).equals("\n")) {
					sd.insertString(sd.getLength(), "\n", null);
				} else {
					if (style == 0) {
						String face = getFace(occ, definition);
						final StringTrameOccView str = new StringTrameOccView(i, editor, face);
						listOfComponents.add(str);
						sd.insertString(sd.getLength(), str.toString(), null);
					} else if (style == 1) {
						Object[] hint_definition = trameController.getFacetDefinition(hintName);
						String face      = getFace(occ, definition);
						String hint_face = getFace(occ, hint_definition);
						final TextFieldTrameOccView label = new TextFieldTrameOccView(i, editor, face, hint_face);
						listOfComponents.add(label);
						label.addFocusListener(new FocusAdapter() {
							public void focusGained(FocusEvent e) {
								focus(new ZoneTrame(-1, -1));
							}
						});
						label.addMouseListener(new MouseAdapter() {
							public void mousePressed(MouseEvent e) {
								if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
									JPopupMenu popup = new JPopupMenu();
									JMenuItem menuSelect = new JMenuItem(Textometre.LABELS.getLocaliseString("wintrame.popup.select.label"));
									menuSelect.setEnabled(true);
									menuSelect.addActionListener(new ActionListener() {
										public void actionPerformed(ActionEvent action) {
											label.select(selection_color);
										}
									});
									try {
										OccurrenceModel myocc  = trameController.getOccurrence(label.getNumocc());
										Action actionReplaceOcc = new ReplaceOccurrenceAction(myocc);
										JMenuItem menuEdit = new JMenuItem(actionReplaceOcc);
										popup.add(menuEdit);
									} catch (Exception err) { }
									popup.add(menuSelect);
									popup.show(label, e.getX(), e.getY());
								}
							}
						});
						editor.insertComponent(label);
					} else {
						ArrayList<String> array = getFaces(occ);
						final ListTrameOccView list = new ListTrameOccView(i, editor, array.toArray());
						listOfComponents.add(list);
						list.addMouseListener(new MouseAdapter() {
							public void mousePressed(MouseEvent e) {
								if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
									JPopupMenu popup = new JPopupMenu();
									JMenuItem menuSelect = new JMenuItem(Textometre.LABELS.getLocaliseString("wintrame.popup.select.label"));
									menuSelect.setEnabled(true);
									menuSelect.addActionListener(new ActionListener() {
										public void actionPerformed(ActionEvent action) {
											list.select(selection_color);
										}
									});
									try {
										OccurrenceModel myocc  = trameController.getOccurrence(list.getNumocc());
										Action actionReplaceOcc = new ReplaceOccurrenceAction(myocc);
										JMenuItem menuEdit = new JMenuItem(actionReplaceOcc);
										popup.add(menuEdit);
									} catch (Exception err) { }
									popup.add(menuSelect);
									popup.show(list, e.getX(), e.getY());
								}
							}
						});
						editor.insertComponent(list);
					}
				}
			}
			editor.setCaretPosition(0);
		} catch (Exception err) {
		}
	}
	/**
	 * Capture les evenements en provenance du cadre (CadreListener).
	 **/
	public void cadreChanged(CadreEvent event) {
		if (event instanceof CadreShowEvent) {
			CadreShowEvent e = (CadreShowEvent)event;
			focus(e.getZone());
		} else if (event instanceof CadreSelectEvent) {
			CadreSelectEvent e = (CadreSelectEvent)event;
			AbstractSelection selections = getSel(e.getArray(), e.getZone());
			add2Selection(selections);
		}
	}

	/**
	 * Capture les evenements en provenance du lexique (LexListener).
	 **/
	public void lexChanged(EventObject event) {
		if (event instanceof LexNameChangedEvent) {
			LexNameChangedEvent e = (LexNameChangedEvent)event;
			String name = e.getName();
			showTnum(name, true, false);
		} else if (event instanceof ReplaceArticleEvent) {
			ReplaceArticleEvent e = (ReplaceArticleEvent)event;
			ReplaceArticleEdit edit = new ReplaceArticleEdit(e.getOldNumap(), e.getOldGraph(), e.getNewGraph(), trameController.getCurrentLexName());
			Textometre.UNDO_SUPPORT.postEdit(edit);
		}
	}

	/**
	 * Capture les evenements en provenance du lexique (LexListener).
	 **/
	public void facetsChanged(FacetsEvent event) {
		String[] options = event.getNames();
		Object selected = facetsComboBox.getSelectedItem();
		if (selected == null) selected = "graph";
		facetsComboBox.removeAllItems();
		for (int i=0; i<options.length; i++) {
			facetsComboBox.addItem(options[i]);
		}
		facetsComboBox.setSelectedItem(selected);
	}


	/********************************************************************
	**   Action de modification d'une occurence
	*********************************************************************/
	public class ReplaceOccurrenceAction extends AbstractAction {
			OccurrenceModel             old_occ;
			OccurrenceModel             new_occ;

		public ReplaceOccurrenceAction(OccurrenceModel current_occ) throws Exception {
			super(Textometre.LABELS.getLocaliseString("action.edit.occurrence"));
			//super(current_occ.getValue(Tnum.GRAPH));
			this.old_occ = current_occ;
		}
		public void replaceOccurrence(OccurrenceModel old_occ, OccurrenceModel new_occ) {
			trameController.replaceOccurence(old_occ, new_occ);
		}
		public void actionPerformed(ActionEvent e) {
			new_occ = trameController.editOccurrence(old_occ);
			if (new_occ != null) {
				UndoableEdit edit = new ReplaceOccurenceEdit(this, old_occ, new_occ);
				Textometre.UNDO_SUPPORT.postEdit(edit);
			}
		}
	}
	/********************************************************************
	** Action reversible de remplacement d'occurrence.
	*********************************************************************/
	public class ReplaceOccurenceEdit extends AbstractUndoableEdit {

			private ReplaceOccurrenceAction action;
			private OccurrenceModel         new_occ;
			private OccurrenceModel         old_occ;
			private String                  currentTnumName;

		public ReplaceOccurenceEdit(Object obj, OccurrenceModel old_occ, OccurrenceModel new_occ) {
			this.action = (ReplaceOccurrenceAction)obj;
			this.old_occ = old_occ;
			this.new_occ = new_occ;
			this.currentTnumName = currentTnumName;
			action.replaceOccurrence(old_occ, new_occ);
		}
		public void undo() throws CannotUndoException {
			action.replaceOccurrence(new_occ, old_occ);
		}
		public void redo() throws CannotRedoException {
			action.replaceOccurrence(old_occ, new_occ);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			return Textometre.LABELS.getLocaliseString("action.replace.occurrence");
		}
	}
	/********************************************************************
	**   Action de concatenation d'une suite d'occurence
	*********************************************************************/
	public class JoinOccurrenceAction extends AbstractAction {
			ArrayList<OccurrenceModel>  old_occs;
			OccurrenceModel             new_occ;
			int                         numstart;

		public JoinOccurrenceAction(TrameModel current_trame, int numstart) {
			super(Textometre.LABELS.getLocaliseString("action.join.occurrences"));
			this.numstart = numstart;
			this.old_occs = new ArrayList<OccurrenceModel>();
			try {
				for (int i=0; i<current_trame.size(); i++) {
					OccurrenceModel occ = trameController.getOccurrence(numstart+i);
					old_occs.add(occ);
				}
			} catch (Exception e) {
				System.err.println("JoinOccurrenceAction: "+e.getMessage());
			}
		}
		public void replaceOccurrence(ArrayList<OccurrenceModel> old_occs, OccurrenceModel new_occ) {
			trameController.replaceOccsByOcc(old_occs, new_occ, numstart);
		}
		public void replaceOccurrence(OccurrenceModel old_occ, ArrayList<OccurrenceModel> new_occs) {
			trameController.replaceOccByOccs(old_occ, new_occs, numstart);
		}
		public void actionPerformed(ActionEvent e) {
			new_occ = trameController.editTrame(old_occs, numstart);
			if (new_occ != null) {
				UndoableEdit edit = new JoinOccurenceEdit(this, old_occs, new_occ);
				Textometre.UNDO_SUPPORT.postEdit(edit);
			}
		}
	}
	/********************************************************************
	** Action reversible de concatenation d'occurrences.
	*********************************************************************/
	public class JoinOccurenceEdit extends AbstractUndoableEdit {

			private JoinOccurrenceAction         action;
			private ArrayList<OccurrenceModel>   old_occs;
			private OccurrenceModel              new_occ;

		public JoinOccurenceEdit(Object obj, ArrayList<OccurrenceModel>  old_occs, OccurrenceModel new_occ) {
			this.action = (JoinOccurrenceAction)obj;
			this.new_occ = new_occ;
			this.old_occs = old_occs;
			action.replaceOccurrence(old_occs, new_occ);
		}
		public void undo() throws CannotUndoException {
			action.replaceOccurrence(new_occ, old_occs);
		}
		public void redo() throws CannotRedoException {
			action.replaceOccurrence(old_occs, new_occ);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			return Textometre.LABELS.getLocaliseString("action.join.occurrences");
		}
	}
	/********************************************************************
	** Action reversible de remplacement d'article.
	*********************************************************************/
	public class ReplaceArticleEdit extends AbstractUndoableEdit {

			private String              old_graph, new_graph;
			private String              currentTnumName;
			private AbstractSelection   sel;

		public ReplaceArticleEdit(int old_numap, String old_graph, String new_graph, String currentTnumName) {
			this.old_graph = old_graph;
			this.new_graph = new_graph;
			this.currentTnumName = currentTnumName;
			this.sel = trameController.getOccs(new Integer(old_numap));

			trameController.changeAllOccurences(sel, new_graph, currentTnumName);
			trameController.notifyTrameChanged();
		}
		public void undo() throws CannotUndoException {
			trameController.changeAllOccurences(sel, old_graph, currentTnumName);
			trameController.notifyTrameChanged();
		}
		public void redo() throws CannotRedoException {
			trameController.changeAllOccurences(sel, new_graph, currentTnumName);
			trameController.notifyTrameChanged();
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			return Textometre.LABELS.getLocaliseString("action.replace.article");
		}
	}
	/********************************************************************
	** Listener pour le menu qui permet de choisir la propriete des items a afficher.
	*********************************************************************/
	private class myItemListener implements ItemListener {
			private String selected;

		public myItemListener(String selected) {
			this.selected = selected;
		}
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() != selected) {
					selected = (String)e.getItem();
					showTnum(selected, false, true);
					maj();
				}
			}
		}
	}
}