/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.util;

import java.io.*;
/*-----------------------------------------------------------------------*/

/** Range est une classe pour representer des intervalles de valeurs de type int.
*/
public class Range implements Serializable {

		private int        minValue, maxValue;
		protected String   title = "text";

	/** Construit un objet Range avec des bornes minimum et maximum.
	 **/
	public Range(int minValue, int maxValue) {
		try {
			if (minValue > maxValue) throw new Exception("minValue is greater than maxValue");
			this.minValue = minValue;
			this.maxValue = maxValue;
		} catch ( Exception e) {
            e.printStackTrace();
        }
	}

	/** Construit un objet Range avec des bornes minimum et maximum.
	 **/
	public Range(String min, String max) {
		try {
			this.maxValue = (Integer.valueOf(max)).intValue();
			this.minValue = (Integer.valueOf(min)).intValue();
			if (minValue > maxValue) throw new Exception("minValue is greater than maxValue");
		} catch ( Exception e) {
            e.printStackTrace();
        }
	}

	/** @return true si la valeur specifiee est comprise dans l'intervalle.
	 **/
	public boolean contains(int value) {
		return ((value >= minValue) && (value <= maxValue));
	}

	/** @return true si l'intervalle specifie est compris dans l'intervalle.
	 **/
	public boolean contains(Range range) {
		return ((range.minValue >= minValue) && (range.maxValue <= maxValue));
	}

	/** @return true si les intervalles partagent leurs deux valeurs extremes.
	 **/
	public boolean equals(Range range) {
		return ((minValue == range.minValue) && (maxValue == range.maxValue) );
	}

	/** @return la borne max de l'intervalle.
	 **/
	public int getMaxValue() {
		return maxValue;
	}

	/** @return la borne min de l'intervalle.
	 **/
	public int getMinValue() {
		return minValue;
	}

	/** @return le nombre d'unites compris dans l'interalle.
	 **/
	public int getLength() {
		return maxValue - minValue + 1;
	}

	/** @return l'intersection des deux intervalles.
	 ** @throws Exception si les deux intervalles n'ont pas d'intersection
	 **/
	public Range intersect(Range range) throws Exception {
		if (intersects(range)) {
			return new Range(
				(minValue < range.minValue)? range.minValue: minValue,
				(maxValue > range.maxValue)? range.maxValue: maxValue
			);
		} else {
			throw new Exception("No intersection between the two ranges");
		}
	}

	/** @return true si les intervalles ont une intersection.
	 **/
	public boolean intersects(Range range) {
		return (
			(contains(range.minValue)) ||
			(contains(range.maxValue)) ||
			(range.contains(minValue)) ||
			(range.contains(maxValue))
		);
	}

	/** @return true si l'intervalle est vide.
	 **/
	public boolean isEmpty() {
		return (minValue == maxValue);
	}

	//Returns the Range of values that are in this Range but not in the given Range.
	//public Range[] subtract(Range range) {
	//}

	/** @return une representation de l'intervalle sous forme de chaine de caracteres.
	 **/
	public String toString() {
		return "["+minValue+".."+maxValue+"]";
	}

	/** @return l'union des deux intervalles.
	 **/
	public Range union(Range range) {
		return new Range(
			(minValue < range.minValue)? minValue: range.minValue,
			(maxValue > range.maxValue)? maxValue: range.maxValue
		);
	}

	/**
	 * Fixe le titre de l'intervalle.
	 **/
	public void setTitle(String title) {
		this.title = title;
	}
	/**
	 * Retourne le titre de l'intervalle.
	 **/
	public String getTitle() {
		return title;
	}
}